package edu.psu.geovista.datamining.data.HD;


/**
 * <p>Title: Interactive Spatial Data Mining for Large and High-Dimensional Data</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: GeoVISTA Center and Department of Geography, Pennsylvania State University</p>
 * @author Diansheng Guo
 * @version 1.0
 */

public class BasicInstance
    implements BasicInstanceInterface {
  private float[] vector;
  private float[] originalvector;
  private int id = -1;
  private Object parent;

  //maxid normally is the total number of instances in the original data
  //this number is useful when some instances are discarded due to null values.
  //to communicate with other components, the references of those discarded
  //instances should be reserved.
  private int maxid = -1;

  //private int[] dims;

  /***
   * Constructor.
   */
  public BasicInstance(int max_id, int id, float[] normalized_vector, float[] originalvalues) {
    //this.dims = dimensions;
    this.maxid = max_id;
    this.id = id;
    this.vector = new float[normalized_vector.length];
    System.arraycopy(normalized_vector, 0, this.vector, 0, this.vector.length);

    this.originalvector = new float[originalvalues.length];
    System.arraycopy(originalvalues, 0, this.originalvector, 0, this.originalvector.length);
  }

  /***
   * Constructor.
   */
  public BasicInstance(int max_id, int id, double[] normalized_vector, double[] originalvalues) {
    //this.dims = dimensions;
    this.maxid = max_id;
    this.id = id;
    this.vector = new float[normalized_vector.length];
    for (int i=0; i< vector.length; i++)
      vector[i] = (float) normalized_vector[i];

    this.originalvector = new float[originalvalues.length];
    for (int i=0; i< vector.length; i++)
      originalvector[i] = (float) originalvalues[i];
  }

  public double getElement(int elementIdx) {
    return this.vector[elementIdx];
  }

  public double getOriginalElementValue(int elementIdx) {
    return this.originalvector[elementIdx];
  }

  public float[] getVector(){
    return this.vector;
  }

  public int getId() {
    return this.id;
  }

  public int getMaxId(){
    return this.maxid;
  }

  public int getNumOfDimensions() {
    return this.originalvector.length;
  }

//  public int[] getDimensions(){
//    return this.dims;
//  }

  public void setParentGroup(Object parentgroup){
    this.parent = parentgroup;
  }

  public Object getParentGroup(){
    return this.parent;
  }

  /**
   * compareTo
   *
   * @param o Object
   * @return int
   */
  public int compareTo(Object o) {
    double value = ( (BasicInstance) o).getKey();
    if (this.getKey() < value)
      return -1;
    else if (this.getKey() > value)
      return 1;
    else
      return 0;
  }

  private double key;
  /**
   * getKey
   *
   * @return double
   */
  public double getKey() {
    return this.key;
  }

  /**
   * setKey
   *
   * @param key double
   */
  public void setKey(double kkey) {
    this.key = kkey;
  }
}
