package edu.psu.geovista.datamining.data;

/**
 * Title:        Algorithms and Modules for Spatial Data Mining
 * Description:  Implementation of dissertation ideas.
 * Copyright:    Copyright (c) 2001
 * Company:      Department of Geography, PennState University
 * @author Diansheng Guo
 * @version 1.0
 */

public class NestedMean
{
  private int     level         = 0;
  private double  currentMean   = -1;

  private NestedMean leftMean;
  private NestedMean rightMean;


  /**
   * Calculate nested means given an array of data...assume values[] are sorted in increasing order
   * */
  public NestedMean(double[] sorted_values, int start_inclusive, int end_inclusive, int current_level, int max_level)
  {
    double sum = 0;
    this.level = current_level;

    for (int i=start_inclusive; i<=end_inclusive; i++)
      sum = sum + sorted_values[i];

    this.currentMean = (sum/(end_inclusive - start_inclusive + 1));

    int i;
    for (i=start_inclusive; i<=end_inclusive; i++)
    {
      if (sorted_values[i] > currentMean) break;
    }

    int meanPosition = i -1;

    if (current_level < max_level)
    {
      this.leftMean  = new NestedMean(sorted_values, start_inclusive, meanPosition,  current_level+1, max_level);
      this.rightMean = new NestedMean(sorted_values, meanPosition +1, end_inclusive, current_level+1, max_level);
    }
  }


  public double[] getMeans(int level_power2)
  {
    int     length  = (int) (Math.pow(2, level_power2)) -1;
    double  means[] = new double [length];

    this.orderNestedValues(means, 0, length);

    for (int i=1; i< means.length; i++)
      if (Double.isNaN(means[i]))
          means[i] = means[i-1];

    return means;
  }

  public int getLevel(){return level;}

  public double getMeanValue()
  {
    return currentMean;
  }

  public NestedMean getLeftNestedMean() { return leftMean; }

  public NestedMean getRightNestedMean() { return rightMean; }



  private void orderNestedValues(double[] means, int left_pos, int right_pos_exclusive)
  {
    int pos = (right_pos_exclusive - left_pos)/2 + left_pos;
    means[pos] = this.currentMean;

    if (pos > left_pos)
    {
      this.leftMean.orderNestedValues(means, left_pos, pos);
      this.rightMean.orderNestedValues(means, pos + 1, right_pos_exclusive);
    }
  }



  public static void main(String[] args)
  {
    double values[] = new double[1000];
    for (int i=0; i< values.length; i++)
      values[i] = i;


    NestedMean nm = new NestedMean(values, 0, values.length - 1, 0, 4);

    double means[] = nm.getMeans(3);

    for (int i=0; i< means.length; i++)
System.out.print("\t" + means[i]);
  }

}