package edu.psu.geovista.datamining.vis.hd;

/**
 * <p>Title: Interactive Spatial Data Mining for Large and High-Dimensional Data</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: GeoVISTA Center and Department of Geography, Pennsylvania State University</p>
 * @author Diansheng Guo
 * @version 1.0
 */

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import edu.psu.geovista.datamining.util.*;
import edu.psu.geovista.ui.event.*;
import edu.psu.geovista.datamining.data.HD.*;

/***
 * This visualization tool uses PCP to view dense cells and possible
 * clusters in high-dimensional space.
 * */
public class HDClusterViewer_New
    extends JPanel
    implements
    /*HighDimensionalClusteringListener,*/
    ColorArrayListener,
    SelectionListener
{
  public static String MODE_NORMAL = "Normal Selection";
  public static String MODE_INDICATION = "Indication";
  public static String MODE_INTERSECT_SEL = "Intersect Sel.";
  public static String MODE_UNION_SEL = "Union Sel.";

  public static String SCALING_NESTED_MEANS ="Axis Scaling: NESTED MEANS";
  public static String SCALING_DATA_MINMAX = "Axis Scaling: DATA MINMAX";
  public static String SCALING_CELL_MINMAX = "Axis Scaling: CELL MINMAX";
  public static String SCALING_GLOBAL_MINMAX = "Axis Scaling: GLOBAL MINMAX";


  public static String VIEW_AGGREGATED_CELLS = "View Aggregated Cells";
  public static String VIEW_SINGLE_DATAITEMS = "View Data Items";

  public boolean debug = false;

  //private DraggablePanel dpanel;
  private PCPCanvas pcp = new PCPCanvas();
  private JScrollPane scrollpane;

  /***
   * Constructor
   * */
  public HDClusterViewer_New() {
    this.setBackground(ColorSetting.background);
    JComboBox modeOptions = new JComboBox(new String[] {
                                             this.MODE_NORMAL,
                                             this.MODE_INTERSECT_SEL,
                                             this.MODE_UNION_SEL,
                                             this.MODE_INDICATION});
    modeOptions.setSelectedIndex(0);
    modeOptions.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox) e.getSource();
        String opt = (String) cb.getSelectedItem();
        changeMode(opt);
      }
    }
    );

    JComboBox scalingOptions = new JComboBox(new String[] {
                                             this.SCALING_NESTED_MEANS,
                                             this.SCALING_DATA_MINMAX,
                                             this.SCALING_CELL_MINMAX,
                                             this.SCALING_GLOBAL_MINMAX});
    scalingOptions.setSelectedIndex(0);
    scalingOptions.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox) e.getSource();
        String opt = (String) cb.getSelectedItem();
        changeScaling(opt);
      }
    }
    );

    JComboBox viewLevelOptions = new JComboBox(new String[] {
                                             this.VIEW_AGGREGATED_CELLS,
                                             this.VIEW_SINGLE_DATAITEMS});
    viewLevelOptions.setSelectedIndex(0);
    viewLevelOptions.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox) e.getSource();
        String opt = (String) cb.getSelectedItem();
        changeViewLevel(opt);
      }
    }
    );

//    dpanel = new DraggablePanel(this);
//    dpanel.setLayout(new FlowLayout());
//    dpanel.add(indicationBox);
//    dpanel.add(nestedSelectionBox);
//    dpanel.add(unionSelectionBox);
//    dpanel.setSize(dpanel.getPreferredSize());
//    dpanel.setBounds(10, 10, dpanel.getWidth(), dpanel.getHeight());
//    dpanel.setOpaque(false);

    pcp.setBackground(ColorSetting.background);
    pcp.setLayout(null);
    JPanel optionPanel = new JPanel(new FlowLayout());
    optionPanel.add(modeOptions);
    optionPanel.add(scalingOptions);
    optionPanel.add(viewLevelOptions);
    this.setLayout(new BorderLayout());
    scrollpane = new JScrollPane(pcp);
    this.add(optionPanel, BorderLayout.NORTH);
    this.add(scrollpane, BorderLayout.CENTER);
    this.repaint();
  }

  public void setOrderredDimensions (int[] orderredDims){
    pcp.setOrderredDimensions(orderredDims);
  }

  private void changeMode(String opt){
    if (opt.compareTo(this.MODE_INDICATION)== 0){
      pcp.setIndication(true);
      pcp.setNestedSelection(false);
      pcp.setUnionSelection(false);
    }else if (opt.compareTo(this.MODE_INTERSECT_SEL)== 0){
      pcp.setIndication(false);
      pcp.setNestedSelection(true);
      pcp.setUnionSelection(false);
    }else if (opt.compareTo(this.MODE_UNION_SEL)== 0){
      pcp.setIndication(false);
      pcp.setNestedSelection(false);
      pcp.setUnionSelection(true);
    }else { //normal
      pcp.setIndication(false);
      pcp.setNestedSelection(false);
      pcp.setUnionSelection(false);
    }
  }

  private void changeScaling(String opt){
    if (opt.compareTo(this.SCALING_NESTED_MEANS)== 0){
      pcp.setAxisScalingMethod(pcp.SCALE_NESTEDMEANS);
    }else if (opt.compareTo(this.SCALING_DATA_MINMAX)== 0){
      pcp.setAxisScalingMethod(pcp.SCALE_DATA_MAX_MIN);
    }else if (opt.compareTo(this.SCALING_CELL_MINMAX)== 0){
      pcp.setAxisScalingMethod(pcp.SCALE_CELL_MAX_MIN);
    }else if (opt.compareTo(this.SCALING_GLOBAL_MINMAX)== 0){
    pcp.setAxisScalingMethod(pcp.SCALE_GLOBAL_MAX_MIN);
    }
  }

  private void changeViewLevel(String opt){
    if (opt.compareTo(this.VIEW_AGGREGATED_CELLS)== 0){
      pcp.setDataLevel(pcp.DATA_LEVEL_HIGH);
    }else if (opt.compareTo(this.VIEW_SINGLE_DATAITEMS)== 0){
      pcp.setDataLevel(pcp.DATA_LEVEL_LOW);
    }
  }

  public void setCells(BasicCell[] cs) {
    if (cs == null)
      return;
    int d = cs[0].getDimensionality();
    //Set a minimum width so that the view won't be two crowed.
    int widthNeeded = 75*d;
    pcp.setPreferredSize(new Dimension(widthNeeded, 300));
    this.remove(scrollpane);
    scrollpane = new JScrollPane(pcp);
    this.add(scrollpane);
    this.validate();
    pcp.setCells(cs);
  }

  public BasicCell[] getCellsWithSyntheticValues(){
    return pcp.getHighLevelCellsWithSyntheticValues();
  }

  public void setAttributeNames(String names[]) {
    pcp.setAttributeNames(names);
  }

  public void colorArrayChanged(ColorArrayEvent e) {
    pcp.repaint();
  }
  public void selectionChanged(SelectionEvent e) {
    pcp.selectionChanged(e);
  }

  /***
   * When density threshold changed, this viewer needs to get the recent dense
   * cells and draw them.
   **/
/*  public void DensityThresholdChanged(HighDimensionalClusteringEvent e) {
    pcp.repaint();
  }

  public void ClusteringChanged(HighDimensionalClusteringEvent e) {
    pcp.repaint();
  }*/

  public synchronized void addSelectionListener(SelectionListener l) {
    pcp.addSelectionListener(l);
  }

  public synchronized void removeSelectionListener(SelectionListener l) {
    pcp.removeSelectionListener(l);
  }

  public synchronized void removeColorArrayListener(ColorArrayListener l) {
    pcp.removeColorArrayListener(l);
  }

  public synchronized void addColorArrayListener(ColorArrayListener l) {
    pcp.addColorArrayListener(l);
  }
}
