#!wish
#
# A Tcl wrapper script for the DCS Graph widget.
#
# Copyright (c) 2005 Natural Diversity Discovery Project.
# All rights reserved.
# 
# Permission is hereby granted, without written agreement and without
# license or royalty fees, to use, copy, modify, and distribute this
# software and its documentation for any purpose, provided that the
# above copyright notice and the following two paragraphs appear in
# all copies of this software.
# 
# IN NO EVENT SHALL THE NATURAL DIVERSITY DISCOVERY PROJECT BE LIABLE 
# TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
# CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE AND 
# ITS DOCUMENTATION, EVEN IF THE NATURAL DIVERSITY DISCOVERY PROJECT
# HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# THE NATURAL DIVERSITY DISCOVERY PROJECT SPECIFICALLY DISCLAIMS ANY 
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
# PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE NATURAL 
# DIVERSITY DISCOVERY PROJECT HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
# SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
#
# @ProposedRating Red (tmcphillips@naturaldiversity.org)
# @AcceptedRating Red (tmcphillips@naturaldiversity.org) 
#
 
# load [Incr Tcl] package
package require Itcl

# import the [Incr Tcl] namespace
namespace import itcl::*

# load the DCSGraph package
package require DCSGraph

# instantiate a DCS::Graph object and store handle
set graph [DCS::Graph .graph -xLabel "x" -yLabel "y"]

# create a list of blt vectors
set vectors {}

# define a handler for processing data from standard input
proc handleInput {} {

    # access the global variable holding the DCS::Graph handle
    global graph  
    
    # access the global variable holding the list of blt vectors 
    global vectors
        
    # read the next command from standard input
    set command [gets stdin]
    
    # exit if command is blank
    if { [llength $command] == 0 } exit
    
    # process the command based on identity of first token in command
    switch [lindex $command 0] {
    
        noop {
            # do nothing
        }
    
        exit { 
            # exit the program
            exit 
        }

        array { 
            # create a blt vector with name specified by the first argument
            eval blt::vector [lindex $command 1] 
            lappend vectors [lindex $command 1]
        }

        clear {
            # delete all of the traces on the graph
            $graph deleteAllTraces
            
            # delete each of the existing blt vectors
            foreach vector $vectors { 
                blt::vector destroy $vector
            }
            
            # reset the list of blt vectors
            set vectors {}
        }

        size {
            wm geometry . [lindex $command 1]x[lindex $command 2]
        }

        windowTitle {
            wm title . [lindex $command 1]
        }

        delete { 
            # destroy the blt vector specified by the first argument 
            blt::vector destroy [lindex $command 1] 
        }

        append { 
            # add the value of the second argument to the blt vector 
            # specified by the first argument
            eval [lindex $command 1] append [lindex $command 2] 
        }

        update { 
            # cause all outstanding graphical operations to 
            # complete immediately
            update idletasks 
        }

        openFile {
            catch { eval $graph $command } 
        }
        
        deleteFile {
            # delete the specified file
            file delete [lindex $command 1]
        }
        
        default { 
            # all other commands are evaluated as methods of 
            # the DCS::Graph instance
            eval $graph $command 
        }
    }
}

# begin generating events when standard input is readable and specify 
# handleInput as the event handler
fileevent stdin readable handleInput
