#
#                        Copyright 2001
#                              by
#                 The Board of Trustees of the 
#               Leland Stanford Junior University
#                      All rights reserved.
#
#                       Disclaimer Notice
#
#     The items furnished herewith were developed under the sponsorship
# of the U.S. Government.  Neither the U.S., nor the U.S. D.O.E., nor the
# Leland Stanford Junior University, nor their employees, makes any war-
# ranty, express or implied, or assumes any liability or responsibility
# for accuracy, completeness or usefulness of any information, apparatus,
# product or process disclosed, or represents that its use will not in-
# fringe privately-owned rights.  Mention of any product, its manufactur-
# er, or suppliers shall not, nor is it intended to, imply approval, dis-
# approval, or fitness for any particular use.  The U.S. and the Univer-
# sity at all times retain the right to use and disseminate the furnished
# items for any purpose whatsoever.                       Notice 91 02 01
#
#   Work supported by the U.S. Department of Energy under contract
#   DE-AC03-76SF00515; and the National Institutes of Health, National
#   Center for Research Resources, grant 2P41RR01209. 
#

package provide DCSUtil 1.0
package require Itcl

class DCS::UniqueName {}
proc DCS::getUniqueName {} { return [string range a[::DCS::UniqueName \#auto] 0 end] }


proc get_next_counter { directory prefix ext } {
    #make sure the directory exists
    if {![file exists $directory]} {
        file mkdir $directory
    }
    if {![file isdirectory $directory]} {
        log_error "$directory is not a directory"
        return 0
    }
    
    #create pattern
    set pattern "${prefix}*"
    if {$ext != "" } {
        set pattern "${prefix}*.${ext}"
    }
    #get file list matches the pattern
    set fileList [glob -nocomplain -types f -directory $directory -- $pattern]

    #find the max counter
    set ll_ext [string length $ext]
    if {$ll_ext} {
        #skip the "." also
        incr ll_ext
    }
    set counter -1
    foreach fileName $fileList {
        #fileName is like test_0_1.jpg test_90_9999.jpg
        #extract the counter part
        if {$ll_ext} {
            set fNoExt [string range $fileName 0 end-$ll_ext]
        } else {
            set fNoExt $fileName
        }
        set ll [string length $fNoExt]
        for {set i 0} {$i < $ll} {incr i} {
            set strIndex [expr "$ll - $i -1"]
            set letter [string index $fNoExt $strIndex]
            if {[lsearch {0 1 2 3 4 5 6 7 8 9} $letter] < 0} {
                break
            }
        }
        if {$i == 0} {
            continue
        }
    
        set c [string range $fNoExt end-[expr $i - 1] end]
        #puts "$c"
    
        set tmp_counter 0
        scan $c "%d" tmp_counter
        
        if { $tmp_counter>$counter } {
            set counter $tmp_counter
        }
    }
    incr counter

    return $counter
}

proc time_stamp {} {
	clock format [clock seconds] -format "%d %b %Y %X"
}


proc isWord { string } {

	regexp {^[ \t]*[^ ]+[ \t]*$} $string

}

######################################################################
# is_positive_int -- returns TRUE if passed string contains a positive
# integer, or a positive integer with white space on either side.
######################################################################

proc isPositiveInt { string } {

	if { [catch {set tempFloat [format "%d" $string]} errorResult ] } {
		return 0
	} else {
		if {$tempFloat < 0} {
			return 0
		} else {
			return 1
		}
	}
}


######################################################################
# is_incomplete_positive_int -- returns TRUE if passed string contains
# a positive integer, just a plus sign, just white space, or any of
# the preceding with white space on either side.
######################################################################

proc isIncompletePositiveInt { string } {

	regexp {^[ \t]*\+?[0-9]*[ \t]*$} $string
}



######################################################################
# is_int -- returns TRUE if passed string contains an integer or an
# integer with white space on either side.
######################################################################

proc isInt { string } {

	if { [catch {set tempFloat [format "%d" $string]} errorResult ] } {
		puts $errorResult
		return 0
	} else {
		return 1
	}
}

proc isField { value } {

	if { [llength $value] > 1} {
		return 0
	} else {
		return 1
	}

}


######################################################################
# is_incomplete_int -- returns TRUE if passed string contains an
# integer, a plus sign, or a minus sign, or any of the preceding with 
# white space on either side.
######################################################################

proc isIncompleteInt { string } {

	regexp {^[ \t]*(\+|\-)?[0-9]*[ \t]*$} $string
}


######################################################################
# is_float -- returns TRUE if passed string contains a float or a float
# with white space on either side.
######################################################################

proc isFloat { string } {
	if { [catch {set tempFloat [format "%f" $string]} errorResult ] } {
		return 0
	} else {
		return 1
	}
}


######################################################################
# is_incomplete_float -- returns TRUE if passed string contains an
# float, a plus sign, or a minus sign, or any of the preceding with 
# white space on either side.
######################################################################

proc isIncompleteFloat { string } {

	regexp {^[ \t]*(\+|\-)?([0-9]*\.?[0-9]*)?[ \t]*$} $string
}


######################################################################
# is_positive_float -- returns TRUE if passed string contains
# a positive float or a positive float with white space on either side.
######################################################################

proc isPositiveFloat { string } {
	
	if { [catch {set tempFloat [format "%12.12f" $string]} errorResult ] } {
		return 0
	} else {
		if {$tempFloat < 0 } {
			return 0
		} else {
			return 1
		}
	}
}


######################################################################
# is_incomplete_positive_float -- returns TRUE if passed string contains
# a positive float, a plus sign, or any of the preceding with 
# white space on either side.
######################################################################

proc isIncompletePositiveFloat { string } {
	set number ""
	set sign ""
	regexp {^[ \t]*\+?([0-9]*\.?[0-9]*)?[ \t]*$} $string sign number
	expr { [isBlank $string] || $sign != "" || $number != "" }
}


######################################################################
# is_blank -- returns TRUE if passed string contains only whate space
######################################################################

proc isBlank { string } {

	regexp {^[ \t]*$} $string
}


proc valuesMatch { value1 value2 type {decimalPlaces 2} {precision 0.01} } {

	# clean up the two values
	set value1 [getCleanValue $value1 $type $decimalPlaces]
	set value2 [getCleanValue $value2 $type $decimalPlaces]

	# return false if either value is invalid
	if { ![valueValid $value1 $type] || ![valueValid $value2 $type] } {
		return 0
	}
	
	# compare values differently based on value type
	switch $type {

		""					{ return [expr [string compare $value1 $value2] == 0] }
		string 			{ return [expr [string compare $value1 $value2] == 0] }
		field 			{ return [expr [string compare $value1 $value2] == 0] }
		int 				{ return [expr [string compare $value1 $value2] == 0] }
		positiveInt 	{ return [expr [string compare $value1 $value2] == 0] }
		float 			{ return [expr abs( double($value1) - double($value2) ) < $precision ] }
		positiveFloat	{ return [expr abs( double($value1) - double($value2) ) < $precision ] }
		default {return -code error "1 Unknown type: $type"}
	}
}


proc valueValid { value type } {

	# call different validation functions based on value type
	switch $type {

		""					{ return 1 }
		string 			{ return 1 }
		field				{ return [isField $value] }
		int 				{ return [isInt $value] }
		positiveInt 	{ return [isPositiveInt $value] }
		float 			{ return [isFloat $value] }
		positiveFloat	{ return [isPositiveFloat $value] }
		default        { return -code error "2 Unknown type: $type" }
	}

	# default is to return false
	return 0
}


proc isIncompleteValue { value type } {

	# call different validation functions based on value type
	switch $type {

		""					{ return 1}
		string 			{ return 1}
		field 			{ return [isField $value]}
		int 				{ return [isIncompleteInt $value] }
		positiveInt 	{ return [isIncompletePositiveInt $value] }
		float 			{ return [isIncompleteFloat $value] }
		positiveFloat	{ return [isIncompletePositiveFloat $value] }
		default {return -code error "3 Unknown type: $type"}
	}

	# return false if data type not recognized
	return 0
}


proc getCleanValue { value type {decimalPlaces 2} } {

	switch $type {

		"" 				{ return $value }
		string 			{ return $value }
		field 			{ return [getCleanField $value] }
		int				{ return [getCleanInt $value] }
		positiveInt		{ return [getCleanInt $value] }
		float				{ return [getCleanFloat $value $decimalPlaces] }
		positiveFloat	{ return [getCleanFloat $value $decimalPlaces] }
		default {return -code error "4 Unknown type: $type"}
	}

	# return origal value if data type not recognized
	return $value
}

proc getCleanField { value } {
	# take the first field only
	return [lindex $value 0]
}


proc getCleanInt { value } {

	# trim off all white space
	set value [string trim $value]

	# trim and save any leading negative sign
	if { [string range $value 0 0] == "-" } {
		set value [string range $value 1 end]
		set sign "-"
	} else {
		set sign ""
	}

	# get length of value before trimming zeros
	set lengthWithLeadingZeros [string length $value]
	
	# trim off leading zeros
	set value [string trimleft $value 0]

	# get length of value after trimming zeros
	set lengthWithoutLeadingZeros [string length $value]

	# set value to zero  if trimming of zeros eliminated all characters
	if { $lengthWithLeadingZeros > 0 && $lengthWithoutLeadingZeros == 0 } {
		set value 0
		set sign ""
	} 

	if {$value == "" } return
	
	# return the cleaned value with sign
	return [expr int(${sign}${value})]
}


proc getCleanFloat { value decimalPlaces } {
	
	if { ! [valueValid $decimalPlaces int ] } {
		return -code error "expected integer for number of decimal places"
	}

	if { [isFloat $value] } {
		
		# reformat value with correct number of decimal places
		set value [format "%.${decimalPlaces}f" $value]

	} else {

		# trim off all white space
		set value [string trim $value]
		
		# trim off leading zeros
		set value [string trimleft $value 0]
	}

	# return the cleaned value
	return $value
}


proc showStack {} {
	
	set currentLevel [info level]
	
	while { $currentLevel > 0 } {
		puts "$currentLevel: [info level $currentLevel]"
		incr currentLevel -1
	}

}

class ::DCS::Units {
	
	private variable scaleFactor
	public method convertUnits {value oldUnits newUnits}
	public method convertUnitValue
	public method getConversionEquation { oldUnits newUnits }
	public method getConversionsForUnits
	private variable conversions

	constructor {} {
		
		set scaleFactor(eV,ev) ""
		
		# set scale factors between length units
		set scaleFactor(mm,mm)	""
		set scaleFactor(um,um) 	""
		set scaleFactor(mm,um)	"1000. *"
		set scaleFactor(um,mm)	"0.001 *"
		
		# set scale factors between angle units
		set scaleFactor(deg,deg)	""
		set scaleFactor(mrad,mrad)	""
		set scaleFactor(deg,mrad)	"17.45329 *"
		set scaleFactor(mrad,deg)	"0.05729578 *"
		set scaleFactor(%,%)       ""
		set scaleFactor(s,s)       ""
		set scaleFactor(s,ms)       "1000.0 * "
		set scaleFactor(ms,s)       "0.001 * "
		set scaleFactor(ms,ms)       ""

        # set scale factors between time
        set scaleFactor(s,min)      "0.016666667 *"
        set scaleFactor(min,s)      "60 *"
        set scaleFactor(min,ms)      "60000 *"
        set scaleFactor(ms,min)      "0.0000166667 *"

		set scaleFactor(counts,counts) ""
		set scaleFactor(V,V)       ""
		
		# set scale factors between energy units
		set scaleFactor(eV,eV)		""
		set scaleFactor(keV,keV)	""
		set scaleFactor(angstrom,angstrom)		""
		set scaleFactor(A,A)       ""
		
		set scaleFactor(eV,keV) 			"0.001 *"
		set scaleFactor(eV,keV) 			"0.001 *"
		set scaleFactor(eV,angstrom)		"12398. /"
		set scaleFactor(eV,A)            "12398. /"

		set scaleFactor(keV,eV)				"1000. *"
		set scaleFactor(keV,angstrom)		"12.398 /"
		set scaleFactor(keV,A)           "12.398 /"

		set scaleFactor(angstrom,eV)		"12398. /"
		set scaleFactor(angstrom,keV)		"12.398 /"
		set scaleFactor(angstrom,A)		""

		set scaleFactor(A,eV)		"12398. /"
		set scaleFactor(A,keV)		"12.398 /"
		set scaleFactor(A,angstrom)		""

		#list of possible conversions
		set conversions(mm) {mm um}
		set conversions(um) {mm um}
		set conversions(ms) {ms s}
		set conversions(s) {ms s}
		set conversions(deg) {deg mrad}
		set conversions(mrad) {deg mrad}
		set conversions(eV) {eV keV A}
		set conversions(keV) {eV keV A}
		set conversions(A) {eV keV A}
		set conversions(%) {%}	
	}
}

body ::DCS::Units::getConversionsForUnits {units_} {
	return $conversions($units_)
}

body DCS::Units::convertUnitValue { value_  toUnits_ } {

	if {$value_ == "" } {return ""}
	
	foreach {value units} $value_ break
	
	if {$units == ""} {return $value}
	
	set convertedValue  [convertUnits $value $units $toUnits_]

	return $convertedValue
}


body ::DCS::Units::convertUnits {value fromUnits toUnits} {
	#guard against weird inputs, don't try to convert...
	if {$fromUnits == ""} {return $value}
	if {$toUnits == ""} {return $value}
	if {$fromUnits == $toUnits } {return $value}

	if { [catch {
		set result [expr $scaleFactor($fromUnits,$toUnits) $value ]
	} errorResult ] } {
		puts "ERROR $errorResult"
		return $value
		return -code error "error converting '$value $fromUnits' to '$toUnits': $errorResult"
	}
	
	return $result
}

body ::DCS::Units::getConversionEquation { fromUnits toUnits} {
	
	if { [catch {
		set result $scaleFactor($fromUnits,$toUnits)
	} errorResult ] } {
		return -code error "error converting '$fromUnits' to '$toUnits': $errorResult"
	}

	return $result
}


#set up a units conversion object
#set up a singleton object mediator called ::units
if { [info commands ::units] == "" } {
	#set up a list of default motors
	::DCS::Units units
}


proc max {num1 num2} {
   if {$num1 > $num2 } {
      return $num1
   } else {
      return $num2
   }
}

proc min {num1 num2} {
   if {$num1 < $num2 } {
      return $num1
   } else {
      return $num2
   }
}



