#!/bin/sh
################################################################################
#     '$RCSfile: nightly-build-script.sh,v $'
#     '$Author: berkley $'
#     '$Date: 2005/08/04 19:25:24 $'
#     '$Revision: 1.31 $'
################################################################################
################################################################################
# This script runs a nightly build for kepler.  It checks kepler against
# the head of the PTII tree.  The process is as follows:
# 1) checkout kepler
# 2) checkout PTII
# 3) compile KEPLER and PTII with ant
# 4) report compile problems
# 5) if there are no compile problems, run kepler junit tests
# 6) report junit problems
# 7) run ptexecute on the list of workflows found in kepler/lib/test-workflows.lst
# 8) report any workflow problems
################################################################################
# set the variables.
CVSUSER=kepler
CVSSERVER=cvs.ecoinformatics.org
CVSREPOSITORY=/cvs
CVSROOT=:ext:$CVSUSER@$CVSSERVER:$CVSREPOSITORY
CVS_RSH=ssh
CVSMODULE=kepler
JAVA_HOME=/usr/java/j2sdk1.4.2_05
ANT_HOME=/usr/local/devtools/jakarta-ant
ANT=$ANT_HOME/bin/ant
PTII=/home/berkley/project/ptolemy
KEPDIR=~kepler/.kepler
WORKDIR=~kepler/$CVSMODULE-nightly-build
BUILDLOG=$WORKDIR/build.log
WFLOG=$WORKDIR/workflow-execute.log
JUNITLOG=$WORKDIR/junit.log
HTMLFILE=$WORKDIR/results.html
EMAILFILE=$WORKDIR/email.txt
CSSFILE=$WORKDIR/style.css
SWFLIST=$WORKDIR/$CVSMODULE/lib/test-workflows.lst
HTMLINSTALL=/var/www/org.kepler-project.www/ROOT/nightly
DISPLAY=:0.0

#set this to true if you don't want to check out or build ptII and Kepler.
#This will only work if you've previously done a full build (i.e. with 
#QUICK set to false) with this script.  This is mostly for testing purposes
QUICK=false

export CVSROOT=$CVSROOT CVS_RSH=$CVS_RSH ANT_HOME=$ANT_HOME JAVA_HOME=$JAVA_HOME PTII=$PTII DISPLAY=$DISPLAY

# if workdir exists, remove it and remake it
if [ "$QUICK" = "false" ]
then
echo REMOVING WORKING DIR $WORKDIR
rm -rf $WORKDIR
echo REMOVING .KEPLER DIR $KEPDIR
rm -rf $KEPDIR
mkdir $WORKDIR
fi
cd $WORKDIR

BEGIN=`date  +'%y%h%d-%T'`
echo BEGIN TIME IS $BEGIN

# Start the HTML
rm $HTMLFILE
touch $HTMLFILE
echo "<html><head><link rel="stylesheet" type="text/css" href="style.css"></head>" >> $HTMLFILE
echo "<body><h1>Kepler Nightly Build System</h1>" >> $HTMLFILE
echo "<hr>" >> $HTMLFILE
echo "<h2>Begin " >> $HTMLFILE
echo $BEGIN >> $HTMLFILE
echo "</h2>" >> $HTMLFILE
echo "<hr>" >> $HTMLFILE

echo "<h2>Jump to...(Red links have build errors, green do not)</h2>" >> $HTMLFILE
echo "<a class=\"coBroken\" href=\"#checkout\">Ptolemy and Kepler Checkouts</a><br>" >> $HTMLFILE
echo "<a class=\"ptBuildBroken\" href=\"#ptbuild\">Ptolemy Build Results</a><br>" >> $HTMLFILE
echo "<a class=\"keplerBuildBroken\" href=\"#keplerbuild\">Kepler Build Results</a><br>" >> $HTMLFILE
echo "<a class=\"jUnitBroken\" href=\"#junit\">Junit Unit Test Results</a><br>" >> $HTMLFILE
echo "<a class=\"workflowBroken\" href=\"#workflow\">Workflow Test Results</a><br>" >> $HTMLFILE
echo "<br>" >> $HTMLFILE

# Start the email file
rm $EMAILFILE
touch $EMAILFILE
echo "Kepler Nightly Build Email Summary" >> $EMAILFILE
echo "For a full report go to http://www.kepler-project.org/nightly" >> $EMAILFILE
echo "-------------------------------------------------------------" >> $EMAILFILE
echo "Build started " $BEGIN >> $EMAILFILE

echo "<a name=\"checkout\"><h3>Checkouts</h3>" >> $HTMLFILE

###################
# checkout kepler #
###################
if [ "$QUICK" = "false" ]
then
  echo CHECKING OUT KEPLER
  cvs -d $CVSROOT co $CVSMODULE
fi

echo "<h3>Kepler successfully checked out.</h3>" >> $HTMLFILE

###################################################### 
# checkout ptolemy. note the password in the cvsroot #
######################################################
CVSROOT=:pserver:anon:@source.eecs.berkeley.edu:/home/cvs/cvsanon
export CVSROOT=$CVSROOT
export CVS_RSH=
cvs login
if [ "$QUICK" = "false" ]
then
  echo CHECKING OUT PTII
  cvs co ptII
fi

echo "<h3>PTII successfully checked out.</h3>" >> $HTMLFILE

# build ptII and kepler
cd $CVSMODULE
# note the hack below....codegen never compiles right.
rm -rf $WORKDIR/ptII/ptolemy/codegen
rm -rf $WORKDIR/ptII/ptolemy/backtrack
export PTII=$WORKDIR/ptII
export KEPLER=$WORKDIR/$CVSMODULE

COBROKEN=#00FF00

#################
# build ptolemy #
#################
echo "<a name=\"ptbuild\"/>" >> $HTMLFILE
if [ "$QUICK" = "false" ]
then
  echo BUILDING PTII
  $ANT -l $BUILDLOG ptolemy
fi
# check to see if it was successful
SUCCESS=`grep "BUILD FAILED" $BUILDLOG`
PTBUILDBROKEN=#00FF00
if [ "$SUCCESS" = "BUILD FAILED" ]
then
  echo PTII BUILD FAILED
  echo "Ptolemy Build Failed: " >> $EMAILFILE
  echo "See http://www.kepler-project.org/nightly/today.html#ptbuild" >> $EMAILFILE
  echo "<h3>Ptolemy Build Results: <em>BUILD FAILED</em> </h3>" >> $HTMLFILE
  PTBUILDBROKEN=#FF0000
else
  echo PTII BUILD SUCCESSFUL
  echo "Ptolemy Build Successful" >> $EMAILFILE
  echo "<h3>Ptolemy Build Results: BUILD SUCCESSFUL</h3>" >> $HTMLFILE
fi

# print the log to the html file no matter what
echo "<pre>" >> $HTMLFILE
cat $HTMLFILE $BUILDLOG >> $HTMLFILE
echo "</pre>" >> $HTMLFILE

################
# build kepler #
################
echo "<a name=\"keplerbuild\"/>" >> $HTMLFILE
if [ "$QUICK" = "false" ]
then
  echo BUILDING KEPLER
  $ANT -l $BUILDLOG install
fi
SUCCESS=`grep "BUILD FAILED" $BUILDLOG`
KEPLERBUILDBROKEN=#00FF00
if [ "$SUCCESS" = "BUILD FAILED" ]
then
  echo KEPLER BUILD FAILED
  echo "Kepler Build Failed: " >> $EMAILFILE
  echo "See http://www.kepler-project.org/nightly/today.html#keplerbuild" >> $EMAILFILE
  echo "<h3>Kepler Build Results: <em>BUILD FAILED</em></h3>" >> $HTMLFILE
  KEPLERBUILDBROKEN=#FF0000
else
  echo KEPLER BUILD SUCCESSFUL
  echo "Kepler Build Successful" >> $EMAILFILE
  echo "<h3>Kepler Build Results: BUILD SUCCESSFUL</h3>" >> $HTMLFILE
fi

# print the log to the html file no matter what
echo "<pre>" >> $HTMLFILE
cat $HTMLFILE $BUILDLOG >> $HTMLFILE
echo "</pre>" >> $HTMLFILE

#######################
# run the junit tests #
#######################
echo RUNNING JUNIT TESTS
echo "<a name=\"junit\"/>" >> $HTMLFILE
echo "<h3>Begin Kepler JUnit Tests</h3>" >> $HTMLFILE
$ANT -l $JUNITLOG test
SUCCESS=`grep "FAILED" $JUNITLOG`
echo "" >> $EMAILFILE
echo "JUnit Tests:" >> $EMAILFILE
JUNITBROKEN=#00FF00
if [ "$SUCCESS" = "" ]
then
  # all unit tests passed
  echo JUNIT TESTS SUCCESSFUL
  echo "All unit tests passed." >> $EMAILFILE
  echo "<h3>All unit tests passed.</h3>" >> $HTMLFILE
else
  # not all unit tests passed
  echo JUNIT TESTS FAILED
  echo "Some or all of the unit tests FAILED: " >> $EMAILFILE
  echo $SUCCESS >> $EMAILFILE
  echo "<em>Some or all of the unit tests FAILED: " >> $HTMLFILE
  echo $SUCCESS >> $HTMLFILE
  echo "</em>" >> $HTMLFILE
  JUNITBROKEN=#FF0000
fi

echo "<h3>Kepler JUnit Log</h3>" >> $HTMLFILE
echo "<pre>" >> $HTMLFILE
cat $HTMLFILE $JUNITLOG >> $HTMLFILE
echo "</pre>" >> $HTMLFILE
echo "<h3>End Kepler JUnit Tests</h3>" >> $HTMLFILE

###############################
# run the ptexecute workflows #
###############################
echo EXECUTING WORKFLOWS
echo "" >> $EMAILFILE
echo "Workflow Tests:" >> $EMAILFILE
echo "For today's workflow test results visit http://www.kepler-project.org/nightly/today.html#workflow" >> $EMAILFILE
echo "<a name=\"workflow\"/>" >> $HTMLFILE
# open the $SWFLIST file and execute each workflow in the list
echo "<h3>Begin Workflow Tests</h3>" >> $HTMLFILE
rm $WORKDIR/workflow.broken
WORKFLOWBROKEN=#00FF00
cat $SWFLIST | while read WF
do
  echo TESTING WORKFLOW $WF
  echo "<h3>testing workflow:" >> $HTMLFILE
  echo $WF >> $HTMLFILE
  echo "</h3>" >> $HTMLFILE
  $ANT -l $WFLOG ptexecute -Dworkflow=$WF
  
  SUCCESS=`grep "Exception" $WFLOG`
  X11ERROR=`grep "InternalError" $WFLOG`
  if [ "$SUCCESS" = "" ] && [ "$X11ERROR" = "" ]
  then
    echo WORKFLOW $WF SUCCESSFUL
    echo "<h2>Workflow $WF threw no exceptions.</h2>" >> $HTMLFILE
  else
    echo WORKFLOW $WF FAILED
    echo $SUCCESS >> $WFLOG.error
    X11ERROR=`grep "X11" $WFLOG.error`
    touch $WORKDIR/workflow.broken
    if [ "$X11ERROR" = "" ]
    then
      # the workflow test failed
      echo "Workflow $WF FAILED: " >> $EMAILFILE
      echo $SUCCESS >> $EMAILFILE
      echo "<em>Workflow test FAILED: " >> $HTMLFILE
      echo "<pre>$SUCCESS</pre>" >> $HTMLFILE
      echo "</em>" >> $HTMLFILE
    else
      # this is a special case where we tried to run a workflow with visual
      # elements...this won't run without a display so it failed.
      echo "<p><em>This workflow is attempting to create visual elements." >> $HTMLFILE
      echo "This workflow may be executing correctly, but ptexecute " >> $HTMLFILE
      echo "cannot find an X Server to use.  DISPLAY is $DISPLAY</em></p>" >> $HTMLFILE
      
      echo "The workflow $WF is attempting to create visual elements" >> $EMAILFILE
      echo "but cannot because the DISPLAY $DISPLAY is not set right or no X Server is running." >> $EMAILFILE
      echo "" >> $EMAILFILE
    fi
  fi
  echo "<pre>" >> $HTMLFILE
  cat $HTMLFILE $WFLOG >> $HTMLFILE
  echo "</pre>" >> $HTMLFILE
done

if [ -a $WORKDIR/workflow.broken ]
then
  WORKFLOWBROKEN=#FF0000
fi

echo "<h3>End Workflow Tests</h3>" >> $HTMLFILE

###########################
# Finish up the HTML file #
###########################
echo FINISHING UP HTML FILES
END=`date  +'%y%h%d-%T'`
echo "<hr>" >> $HTMLFILE
echo "<h2>End " >> $HTMLFILE
echo $END >> $HTMLFILE
echo "</h2>" >> $HTMLFILE
echo "<hr>" >> $HTMLFILE
echo "</body></html>" >> $HTMLFILE

# Install the HTML file into the web server directory and rotate the log
cp $HTMLFILE $HTMLINSTALL/$END.html
cp $HTMLINSTALL/$END.html $HTMLINSTALL/today.html
echo "" >> $EMAILFILE
echo "------------------------------------------" >> $EMAILFILE
echo "Build ended at $END" >> $EMAILFILE

# write out the CSS.  this is done dymanically so the quick links can be colored red and green
echo WRITING OUT CSS
echo "em {" >> $CSSFILE
echo "  color: #FF0000;" >> $CSSFILE 
echo "  font-weight: bold;" >> $CSSFILE
echo "  font-style:  normal;" >> $CSSFILE
echo "}" >> $CSSFILE

echo ".coBroken {" >> $CSSFILE
echo "  color: $COBROKEN;" >> $CSSFILE
echo "}" >> $CSSFILE

echo ".ptBuildBroken {" >> $CSSFILE
echo "  color: $PTBUILDBROKEN;" >> $CSSFILE
echo "}" >> $CSSFILE

echo ".keplerBuildBroken {" >> $CSSFILE
echo "  color: $KEPLERBUILDBROKEN;" >> $CSSFILE
echo "}" >> $CSSFILE

echo ".jUnitBroken {" >> $CSSFILE
echo "  color: $JUNITBROKEN;" >> $CSSFILE
echo "}" >> $CSSFILE

echo ".workflowBroken {" >> $CSSFILE
echo "  color: $WORKFLOWBROKEN;" >> $CSSFILE
echo "}" >> $CSSFILE
# install the CSS file
cp $CSSFILE $HTMLINSTALL/style.css

# Send out the email
echo SENDING EMAIL
$ANT nightlybuild.notify -Dattach=$EMAILFILE -Ddate=$END

echo DONE AT $END
