/* Author: Michael Shapiro. US Army CERL.

 */



#include <stdio.h>

#include <stdlib.h>

#include <string.h>

#include <time.h>





#define GLOBAL

#include "buffer.h"

#include "distance.h"

#include "local_proto.h"

#include "org_ecoinformatics_seek_gis_grass_BufferJniGlue.h"



int GISBuffer(char *raster_name,int nrows,int ncols, char *buffer_name, int nd,float *pd)

{

	window.cols=nrows;

	window.rows=ncols;

	window.proj=PROJECTION_XY;

	ns_to_ew_squared=1.0;

	ndist=nd;

	map= (MAPTYPE *) calloc (nrows*ncols, sizeof(MAPTYPE));

	distances = (struct Distance *) calloc (nd, sizeof(struct Distance));

	for(int i=0;i<nd;i++)

	{

		printf("Buffering Distance: %10d %10.5f\n",i,pd[i]);

		distances[i].dist=pd[i]*pd[i]; 

	}

	FILE *fr,*fb;

	if((fr=fopen(raster_name,"r"))==NULL)

	{

		printf("can not open raster file %s\n",raster_name);

		return(-1);

	}

	minrow = -1; maxrow = -1;

    mincol = ncols; maxcol = 0;



	clock_t tb=clock();



	MAPTYPE c;

	for(i=0;i<nrows;i++)

	{

		for(int j=0;j<ncols;j++)

		{

			fscanf(fr," %d ",&c);

			*(map + MAPINDEX(i, j))=c;

			//printf("%1d",c);

			if(c!=0)

			{

				if (minrow < 0) minrow = i;

				maxrow = i;

				if (j < mincol) mincol = j;

				if (j > maxcol) maxcol = j;

			}

		}

		//printf("\n");

	}

	fclose(fr);

	execute_distance(1);

	//printf("Buffering (not include output) takes %10.3f seconds\n",(double)(clock()-tb)/CLOCKS_PER_SEC);

	//getch();

	//exit(0);



	if((fb=fopen(buffer_name,"w"))==NULL)

	{

		printf("can not open buffer file %s\n",buffer_name);

		return(-1);

	}

	for(i=0;i<nrows;i++)

	{

		for(int j=0;j<ncols;j++)

			fprintf(fb,"%3d",*(map + MAPINDEX(i, j)));

		fprintf(fr,"\n");

	}

	fclose(fb);

	free(map);

	free(distances);

	return(0);

}



JNIEXPORT jint JNICALL Java_org_ecoinformatics_seek_gis_grass_BufferJniGlue_GISBuffer

  (JNIEnv * env, jobject, jstring s1, jint i1, jint i2, jstring s2, jint i3, jfloatArray fa)

{

  printf("Calling GISBuffer in CPP Code.\n");

  jboolean iscopy;

  char * rfile = (char*) env->GetStringUTFChars(s1, &iscopy);

  char * bfile = (char*) env->GetStringUTFChars(s2, &iscopy);

  jfloat *pd = env->GetFloatArrayElements(fa,0);

  printf("calling GISBuffer now\n");

  jint numh=GISBuffer(rfile,i1,i2,bfile,i3,pd);

  env->ReleaseFloatArrayElements(fa, pd, 0);

  printf("DONE Calling GISBuffer in CPP Code.\n");

  return(numh);

}



