<!-- This file is the second file in the process of Actor Documentation. 

     This file produces all the possible actors in Kepler using annotations.owl as

     well as the basicActorKeplerLibrary.xml. Output produced as ConceptActorFormation.xml -->







<!--

			### Documentation Process ###



   1)  Process ontology.owl to KeplerConcepts.xml  [xsl stylesheet: ConceptFormation.xsl]

   2)  Process KeplerConcepts.xml to ConceptActorOutput.xml [xsl stylesheet: ActorFormation.xsl]

   3)  Process ConceptActorOutput.xml to KeplerDocumentation.html [xsl stylesheet: DocumentationFormation.xsl]



   Using Ant:-

   > ant build.xml



   Using makefile:-

   > make documentation



   File Output Contains:- 1) The concept tree hierarchy structure (same as 1st step)

			  2) A List of all possible Concepts  ( same as 1st step)

			  3) A List of all the possible actors (actor nodes obtained from files such as annotation.owl & basicKeplerActorLibrary.xml)



-->     	







<xsl:stylesheet version="1.0"

xmlns:xsl="http://www.w3.org/1999/XSL/Transform"

xmlns:rdf  = "http://www.w3.org/1999/02/22-rdf-syntax-ns#"

xmlns:rdfs = "http://www.w3.org/2000/01/rdf-schema#"

xmlns:owl  = "http://www.w3.org/2002/07/owl#"

xmlns:ont  = "http://seek.ecoinformatics.org/ontology#"

xmlns      = "http://seek.ecoinformatics.org/ontology#"

xml:base   = "http://seek.ecoinformatics.org/ontology#"



xmlns:xsd  = "http://www.w3.org/2001/XMLSchema#"

xmlns:asc  = "http://seek.ecoinformatics.org/annotation-schema#"

xmlns:ano  = "http://seek.ecoinformatics.org/annotations#">





<xsl:output method="xml" doctype-system="http://ptolemy.eecs.berkeley.edu/xml/dtd/MoML_1.dtd" 

     doctype-public="-//UC Berkeley//DTD MoML 1//EN" />



<!-- Produce all the nodes for conceptNames -->

<xsl:template match="ont:keplerConcepts">

	<keplerConcepts>

		<xsl:apply-templates/>

	</keplerConcepts>

</xsl:template>



<!-- ============================================================================================== -->

<!-- To make the concept hierarchy tree in the final documentation for clarity of concept structure -->

<xsl:template match="ont:keplerConcepts/ont:conceptTree">



<conceptTree>

		<xsl:for-each select="ont:ParentConcept">

			<parentConcept>

				<xsl:value-of select="node()"/>

				<xsl:for-each select="ont:ChildConcept">

					<childConcept>

						<xsl:value-of select="."/>

					</childConcept>

				</xsl:for-each>

			</parentConcept>

		</xsl:for-each>





</conceptTree>



</xsl:template>





<!-- ============================================================================================== -->

<!-- using Actor Files namely annotations.owl and basicActorKeplerLibrary.xml produce actor nodes -->



<!-- Both annotations.owl and basicActorKeplerLibrary.xml consist of same actors, however basicActorKepler

     Library.xml consists of further actor information such as Class,EntityName etc. 

-->



<xsl:template match="ont:keplerConcepts/ont:ActorFiles">



	<!-- using annotations.owl-->

	<xsl:for-each select="document(@name)/rdf:RDF/asc:Annotation">



		<ActorName>

			<ActorConcept>

				<xsl:value-of select="asc:annotates/asc:ItemTag/rdf:type/@rdf:resource"/>

			</ActorConcept>

			<ActorLsid>

				<xsl:value-of select="asc:annotates/asc:ItemTag/asc:taggedItem/asc:Actor/asc:lsid"/>

			</ActorLsid>

		</ActorName>



	</xsl:for-each>



	<!-- using basicActorKeplerLibrary.xml -->

	<xsl:for-each select="document(@name)/group/entity">

		<ActorEntity>



			<EntityName>

				<xsl:value-of select="@name"/>

			</EntityName>



			<EntityClass>

				<xsl:value-of select="@class"/>

			</EntityClass>

	

			<ActorLsid>

				<xsl:value-of select="property/@value"/>

			</ActorLsid>





		</ActorEntity>

	</xsl:for-each>





</xsl:template>





<!-- ============================================================================================== -->

<!-- Listing all the possible main and sub concepts in Kepler -->



<xsl:template match="ont:keplerConcepts/ont:conceptNameMain">

	<conceptName>http://seek.ecoinformatics.org/ontology<xsl:value-of select="."/>

	</conceptName>

</xsl:template>



<xsl:template match="ont:keplerConcepts/ont:conceptName">

	<conceptName>http://seek.ecoinformatics.org/ontology#<xsl:value-of select="."/>

	</conceptName>

</xsl:template>







</xsl:stylesheet>