<!-- This file is the third and final stylesheet involved in the documentation process.

     It gets the concepts and actors involved in kepler from the preceding ConceptActorOutput.xml

     file produced. The resultant output file is the html documentation of various kepler concepts and 

     actors. -->





<!--

			### Documentation Process ###



   1)  Process ontology.owl to KeplerConcepts.xml  [xsl stylesheet: ConceptFormation.xsl]

   2)  Process KeplerConcepts.xml to ConceptActorOutput.xml [xsl stylesheet: ActorFormation.xsl]

   3)  Process ConceptActorOutput.xml to KeplerDocumentation.html [xsl stylesheet: DocumentationFormation.xsl]



   Using Ant:-

   > ant build.xml



   Using makefile:-

   > make documentation



   This file output contains:- 1) Same information as obtained in the 2nd Step Process of ActorFormation, however

				  information is now formatted for display in an HTML form of output.

	



-->     	





<xsl:stylesheet version="1.0"

xmlns:xsl  = "http://www.w3.org/1999/XSL/Transform"

xmlns:xsd  = "http://www.w3.org/2001/XMLSchema#"

xmlns:owl  = "http://www.w3.org/2002/07/owl#"

xmlns:rdf  = "http://www.w3.org/1999/02/22-rdf-syntax-ns#"

xmlns:rdfs = "http://www.w3.org/2000/01/rdf-schema#"

xmlns:ont  = "http://seek.ecoinformatics.org/ontology#"

xmlns:asc  = "http://seek.ecoinformatics.org/annotation-schema#"

xmlns:ano  = "http://seek.ecoinformatics.org/annotations#"

>



<xsl:output method="html" omit-xml-declaration="yes"/>



<xsl:template match="ont:keplerConcepts">

	<html>

		<body>







			<!-- First Main Heading : Kepler Concepts and Actors --> 

			<H1 align="center">

				<font color="DarkBlue">

					<b>

					Kepler Concepts and Actors

					<HR/>

					</b>

				</font>

			</H1>

			

			<!-- ======================================================================================= -->

			<!-- concepts heirarchy list -->

			<ul>

				<p/>

				

				<!-- Second Sub Heading	: Concept Hierarchy Tree -->						

				<H2><font color="DarkBlue">Concept Hierarchy Tree</font></H2>





				<!-- List of Concepts an Sub Concepts in Kepler-->	

				<xsl:for-each select="ont:conceptTree/ont:parentConcept">

		

					<xsl:if test="./ont:childConcept">

						<xsl:variable name="thisParentConcept" select="node()"/>

						



						<li>

							<!-- Setting a hyperlink for the main concepts in the list -->

							<xsl:variable name="thisLink" select="concat('#',$thisParentConcept)"/>

							<a href="{$thisLink}">

					

							<b><xsl:value-of select="$thisParentConcept"/></b></a>

					

							<BLOCKQUOTE>

								<xsl:for-each select="ont:childConcept">

									<xsl:value-of select="."/><br/>

								</xsl:for-each>	

							</BLOCKQUOTE>

					

						</li>

					

					</xsl:if>

					<p/>

				</xsl:for-each>

			</ul>



			<HR/><HR/><p/><p/>









			<!-- ======================================================================================= -->



			<!-- available concepts list -->

			<font color="DarkBlue" size="4"><H2 align="center"><u>

				Available Concepts and Their Actors

				</u></H2>

			</font>



			

			<!-- getting all unique concepts in List -->

			<xsl:variable name="conceptList" select="ont:conceptName[not(.=preceding::ont:conceptName)]"/>

		



				

					<xsl:for-each select="$conceptList">



						<xsl:variable name="conceptThis" select="."/>

						<font size="4">

							<xsl:variable name="concept" select="substring-after(.,'#')"/>

							<a name="{$concept}"/>

							<b><xsl:value-of select="$concept"/></b>

							



						</font>



						<!-- Find all the actors that belong to the current concept -->						

						<xsl:for-each select="//ont:ActorName">

							<xsl:variable name="actorThis" select="."/>

	

							<xsl:if test="ont:ActorConcept=$conceptThis">

								<p/>

								<BLOCKQUOTE>

									<xsl:for-each select="//ont:ActorEntity">

										<xsl:if test="concat($conceptThis,ont:ActorLsid) =$actorThis">

		

											<b><font color="DarkBlue">

												<xsl:value-of select="ont:EntityName"/></font>

											</b>

											<br/> 

											<ul type="CIRCLE">

												<b><li>Class:</li></b>

												<xsl:value-of select="ont:EntityClass"/>

												<br/>

												<b><li>LSID:</li></b>

												<xsl:value-of select="ont:ActorLsid"/>

											</ul>





										</xsl:if>

									</xsl:for-each>

								 

								</BLOCKQUOTE>

							</xsl:if>						



						 </xsl:for-each>

						<p/>

						



					</xsl:for-each>

				 







				 <HR/>

		</body>	

	</html>

</xsl:template>





<xsl:template match="*" priority="-1.0">

<TEST.MISSED/>

</xsl:template>





</xsl:stylesheet>