/* A thread for executing the lidar processing.

*/



package org.geon;



import java.util.*;

import java.sql.*;



//////////////////////////////////////////////////////////////////////////

//// ExecutionThread

/**

* Thread for executing the Lidar processing.

@author Efrat Jaeger

*/

public class LidarUtilities {



    private static final String METADATATABLE = "NSAF.META";

    public final long QUERYLIMIT = 1600000;

//    private long numRows = -2;



    public LidarUtilities(StringBuffer threadResp, String header, String footer, String srid) {

        this.threadResp = threadResp;

        this.header = header;

        this.footer = footer;

	this.srid = srid;

    }



    public StringBuffer threadResp;

    private String header;

    private String footer;

    private String srid;



    public long calculateNumRows(String MinX, String MinY, String MaxX, String MaxY,

				 String[] classification) {

        Connection con = null;

        try {

            con = connect();



        } catch (Exception ex) {

            System.out.println("unable to connect to lidar database");

            ex.printStackTrace();

            threadResp.append(header);

            threadResp.append("<tr><td><h2>Error!<h2></td></tr>");

            threadResp.append("<tr><td>Unable to connect to the lidar database");

            threadResp.append("</td></tr>");

            threadResp.append(footer);

            disconnect(con);

            return -1;

        }

        //Get all tables within the bounding box.

        tableNames = new Vector();

        tableNames = getResidingTableNames(con, MinX, MaxX, MinY, MaxY);

        if (tableNames.size() == 0) {

            disconnect(con);

            return -1;

        }

        // create the query constraint.

        StringBuffer constraint = createConstraint(classification, MinX, MinY, MaxX, MaxY);



        //sum the number of rows by each table

        long count = countAcrossTables(con, tableNames, constraint);



        disconnect(con);

        return count;

    }



    public Vector getTableNames(String MinX, String MinY, String MaxX, String MaxY) {

        Vector tNames = new Vector();

        Connection con = null;

        try {

            con = connect();



        } catch (Exception ex) {

            System.out.println("unable to connect to lidar database");

            ex.printStackTrace();

            threadResp.append(header);

            threadResp.append("<tr><td><h2>Error!<h2></td></tr>");

            threadResp.append("<tr><td>Unable to connect to the lidar database");

            threadResp.append("</td></tr>");

            threadResp.append(footer);

            disconnect(con);

            return tNames;

        }

        //Get all tables within the bounding box.

        tNames = getResidingTableNames(con, MinX, MaxX, MinY, MaxY);

        disconnect(con);

        return tNames;

    }





    private long countAcrossTables(Connection con, Vector tableNames, StringBuffer constraint) {

        long count = 0;

        for (int i=0; i<tableNames.size(); i++) {

            long tmpCount = countQuery(con, constraint.toString(), (String)tableNames.get(i));

	        if (tmpCount == -1) {

	            return -1;

	        }

	        count += tmpCount;

            if (count > QUERYLIMIT)

                return count;

        }

        return count;

    }



    private Connection connect() throws Exception {

        Connection con = null;

        Class.forName("com.ibm.db2.jcc.DB2Driver").newInstance();

        con = DriverManager.getConnection("jdbc:db2://ds003.sdsc.edu:60024/LIDAR","efrat","arf_usa1");

        return con;

    }



    private void disconnect(Connection con) {

        try {

            con.close();

        } catch (Exception ex) {

            con = null;

        }

    }



    private Vector getResidingTableNames(Connection con, String X1, String X2, String Y1, String Y2) {

        String query = "select table_name from " + METADATATABLE;

        query += "\nwhere ";

        query += "(((" + X1 + " <= X_MIN) and (X_MIN <= " + X2 + ")) " +

                "or ((" + X1 + " <= X_MAX) and (X_MAX <= " + X2 + ")) " +

                "or ((X_MIN <= " + X1 + ") and (" + X2 + " <= X_MAX))) \n" +

                "and (((" + Y1 + " <= Y_MIN) and (Y_MIN <= " + Y2 + ")) " +

                "or ((" + Y1 + " <= Y_MAX) and (Y_MAX <= " + Y2 + ")) " +

                "or ((Y_MIN <= " + Y1 + ") and (" + Y2 + " <= Y_MAX))) ";

        Vector tablesVec = new Vector();

        try{

            Statement st = con.createStatement();

            ResultSet rs = st.executeQuery(query);



            while (rs.next()) {

                String val = rs.getString(1);

                if (val != null && !val.equals("")) {

                    System.out.println("tableName  = " + val);

                    tablesVec.add(val);

                }

            }

            rs.close();

            if (tablesVec.size() > 0) {

                return tablesVec;

            }

        } catch (Exception e1) {

            System.out.println("Unable to query lidar database table " + METADATATABLE);

            e1.printStackTrace();

            threadResp.append(header);

            threadResp.append("<tr><td><h2>Error!<h2></td></tr>");

            threadResp.append("<tr><td>Unable to query the lidar database");

            threadResp.append("</td></tr>");

            threadResp.append(footer);

            return tablesVec;

        }

        // tablesVec size is empty.

        threadResp.append(header);

        threadResp.append("<tr><td><h2>Empty Query Response!<h2></td></tr>");

        threadResp.append("<tr><td>Querying for ");

        threadResp.append("bounding box selection: MinX = " + X1 + ", MaxX = " + X2 + ", ");

        threadResp.append("MinY = " + Y1 + ", MaxY = " + Y2 + " returned no result!</td></tr>");

        threadResp.append(footer);

        return tablesVec;

    }



    public StringBuffer createConstraint(String[] classification, String MinX, String MinY, String MaxX, String MaxY) {

        StringBuffer constraint = new StringBuffer();

        constraint.append("WHERE ");

        if (classification != null) {

            if (classification.length > 0 && classification.length < 4) {// something was selected but not all

                constraint.append("(CLASSIFICATION = '" + classification[0] + "'");

                for (int i=1; i<classification.length; i++) {

                     constraint.append(" OR CLASSIFICATION = '" + classification[i] + "'");

                }

                constraint.append(") ");

                constraint.append("AND ");

            }

        }

        constraint.append("db2gse.EnvelopesIntersect ( geometry, ");

        constraint.append(MinX + ", " + MinY + ", " + MaxX + ", " + MaxY + ", " + srid + " ) = 1");

        System.out.println("constraint in LidarUtil: " + constraint.toString()); 

	return constraint;

    }



    private long countQuery(Connection con, String constraint, String tableName) {



        String query = "select count(*) from " + tableName + "\n" + constraint;

        System.out.println("count query ==> " + query);

        try{

            Statement st = con.createStatement();

            ResultSet rs = st.executeQuery(query);



            while (rs.next()) {

                String val = rs.getString(1);

                if (val == null || val.equals("")) {

                    throw new Exception("query value is null");

                } else {

                    long count = Long.parseLong(val);

                    System.out.println("number of rows per table " + tableName + " ==> " + count);

                    return count;

                }

            }

            rs.close();



        } catch (Exception e1) {

            System.out.println("unable to query lidar database table " + tableName);

            e1.printStackTrace();

            threadResp.append(header);

            threadResp.append("<tr><td><h2>Error!<h2></td></tr>");

            threadResp.append("<tr><td>Unable to query the lidar database");

            threadResp.append("</td></tr>");

            threadResp.append(footer);

            return -1;

        }

        return -1;

    }



    static public void main(String args[]) {

        String[] classification = {"G"};

        StringBuffer threadResp = new StringBuffer();

        LidarUtilities et = new LidarUtilities(threadResp,"","","1005");

        long count = et.calculateNumRows("6101117", "1971306", "6207459", "1991991", classification);

        for (int i=0; i<et.tableNames.size(); i++) {

            System.out.println((String)et.tableNames.get(i));

        }

        System.out.println("number of rows ==> " + count);

    }



    public long estimateTime(long nPoints) {

	double q = queryTime(nPoints);

	double p = processTime(nPoints);

	System.out.println(nPoints + " points, query time " + q + ", process time " + p);  

	return Math.round(p + q);

	//return Math.round(queryTime(nPoints) + processTime(nPoints));

    }

	

    public double processTime(long nPoints) {

	return Math.pow(10,-9) * Math.pow(nPoints,2) + 0.0003 * nPoints + 50.492;

    }



    public double queryTime(long nPoints) {

	return -8 * Math.pow(10,-12) * Math.pow(nPoints,2) + 5 * Math.pow(10,-5) * nPoints + 15.399;

    }



    public Vector tableNames;

}

