/* GriddlesExec actor subclased from stanard Exec from PtolemyII concepts.
 Copyright (c) 2005 The Regents of the University of California.
 All rights reserved.

  Permission is hereby granted, without written agreement and without
 license or royalty fees, to use, copy, modify, and distribute this
 software and its documentation for any purpose, provided that the above
 copyright notice and the following two paragraphs appear in all copies
 of this software.

 IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

 THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.

                                        PT_COPYRIGHT_VERSION_2
                                        COPYRIGHTENDKEY


*/

package org.monash.griddles;

import javax.xml.rpc.Stub;

import org.monash.griddles.jobrun.JobrunIF;
import org.monash.griddles.jobrun.MyJobrun_Impl;

import ptolemy.actor.TypedIOPort;
import ptolemy.data.StringToken;
import ptolemy.data.expr.Parameter;
import ptolemy.data.expr.StringParameter;
import ptolemy.data.type.BaseType;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.InternalErrorException;
import ptolemy.kernel.util.NameDuplicationException;

/**
  *   GriddlesRemoteExec is a subclass of the GriddlesExec actor in 
  *   which we split functionality of above actor into two parts and 
  *   merged one part into web service implementation. We added web 
  *   service client functionality into this actor, which allows 
  *   communication between this actor and the web service. This actor 
  *   could be configured from the Kepler user interface for executing 
  *   command at any specified remote location. In addition a certificate 
  *   based security component has been implemented to make communication 
  *   between this actor and the web service fully secure. This actor encodes 
  *   the data with the public key of the web service and signs the data with 
  *   its private key before sending a request to the web service. The web 
  *   service on the other end opens the request by using a client public key 
  *   and decodes by using its private key. The web service executes the command 
  *   with the arguments that were received from the client. The stderr and stdout 
  *   messages generated by the execution are sent in response to the request. 
  *   The response is also encoded and signed with certificates as before. 
  *   The actor upon receiving response opens, decodes and creates stderr and stdout 
  *   objects. Subsequently these messages are streamed to respective ports like previous actor.
  *
  * 
  * This actor defines a new TypedIOPort for subclasses to use especially JGridletRemoteCreate ...   
  * @author Jagan Kommineni, Monash University July 2005
  * @version $Id: GriddlesRemoteExec.java,v 1.3 2005/11/01 20:39:13 ruland Exp $
  **/


public class GriddlesRemoteExec extends GriddlesExec {
  public GriddlesRemoteExec(CompositeEntity container, String name)
    throws NameDuplicationException, IllegalActionException  {
    super(container, name);
    executionHost = new Parameter(this, "executionHost", new StringToken("brecca-2.vpac.org"));
    webServicePort = new Parameter(this, "webServicePort", new StringToken("8080"));
    executionHost.setStringMode(true);
    webServicePort.setStringMode(true);
    }
    
      public StringParameter newStringParameter(String name) throws NameDuplicationException {
    try {
      _workspace.getWriteAccess();
      StringParameter param = new StringParameter(this, name);
      return param;
      } 
    catch (IllegalActionException ex) {
      // This exception should not occur, so we throw a runtime
      // exception.
      // throw new InternalErrorException(this, ex, null);
      } 
    finally {
      _workspace.doneWriting();
      }
    return null;
    }    
  /** Create a new TypedIOPort with the specified name.
    *  The container of the port is set to this actor.
    *  This method is write-synchronized on the workspace.
    *
    *  @param name The name for the new port.
    *  @return The new port.
    *  @exception NameDuplicationException If the actor already has a port
    *   with the specified name.
    */
    
  public ptolemy.kernel.Port newPort(String name) throws NameDuplicationException {
    try {
      _workspace.getWriteAccess();
      TypedIOPort port = new TypedIOPort(this, name);
      port.setTypeEquals(BaseType.STRING);
      return port;
      } 
    catch (IllegalActionException ex) {
      // This exception should not occur, so we throw a runtime
      // exception.
      throw new InternalErrorException(this, ex, null);
      } 
    finally {
      _workspace.doneWriting();
      }
    }
      
  public Parameter executionHost;
  public Parameter webServicePort;
  
  private static Stub createProxy() {
    // Note: MyJobrun_Impl is implementation-specific.
    return (Stub)(new MyJobrun_Impl().getJobrunIFPort());
    }
     /****** 
     The web service stub calls the web service to execute a command on remote 
     location and gets back stderr and stdout messages and these messages are
     streamed to the error and output ports respectively
     Web serves and clients developed by using jwsdp from sun .....
     *******/
  public synchronized void fire() throws IllegalActionException {
    try{
      String serviceURL="http://"+(((StringToken)executionHost.getToken()).stringValue())+
                        ":"+(((StringToken)webServicePort.getToken()).stringValue())+
                        "/jobrun-jaxrpc/jobrun";
      System.out.println("Service URL=" + serviceURL);
      
      
      Stub stub = createProxy();

      stub._setProperty(
      javax.xml.rpc.Stub.ENDPOINT_ADDRESS_PROPERTY,
      serviceURL);
      JobrunIF jobrun = (JobrunIF)stub;
  
      String stdErrorOut = jobrun.jobrun(((StringToken)command.getToken()).stringValue());
      //   System.out.println("Error message:"+ stdErrorOut.getStderror());
      //   System.out.println("Output message:"+ stdErrorOut.getStdout());
      if(stdErrorOut == null)
              stdErrorOut ="%-%No results returned from server";
      System.out.println("ErrorOut="+stdErrorOut);
      
      String[] stringArray = stdErrorOut.split("%-%");
        
      error.send(0, new StringToken(stringArray[0]));
      output.send(0, new StringToken(stringArray[1]));
      }
    catch(Exception ex) {  System.out.println("The exception raised");}
    }
    
       /******* 
         This method stops execution at the end of first iteration 
        *******/
    
        public boolean postfire() 
          {
          return false;
          }
  }