/* Handler for Nexus taxa blocks.
 *
 * Copyright (c) 2005 Natural Diversity Discovery Project.
 * All rights reserved.
 * 
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 * 
 * IN NO EVENT SHALL THE NATURAL DIVERSITY DISCOVERY PROJECT BE LIABLE 
 * TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR 
 * CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE AND 
 * ITS DOCUMENTATION, EVEN IF THE NATURAL DIVERSITY DISCOVERY PROJECT
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE NATURAL DIVERSITY DISCOVERY PROJECT SPECIFICALLY DISCLAIMS ANY 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE NATURAL 
 * DIVERSITY DISCOVERY PROJECT HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

   @ProposedRating Red (tmcphillips@naturaldiversity.org)
   @AcceptedRating Red (tmcphillips@naturaldiversity.org) 
 */
 
package org.nddp.phylogeny;

import java.util.Iterator;

import org.nddp.Collection;
import org.nddp.CollectionManager;
import org.nddp.NewCollectionManager;
import org.nddp.exceptions.ParseException;

import ptolemy.data.IntToken;
import ptolemy.data.StringToken;
import ptolemy.kernel.util.IllegalActionException;

class NexusTaxaBlockHandler extends NexusBlockHandler {

    public NexusTaxaBlockHandler(NexusBlock block, CollectionManager 
        nexusFileCollectionManager) throws IllegalActionException {
        
		super(block, nexusFileCollectionManager);
        
		Collection taxaCollection = new TaxonCollection("");
		_taxaCollectionManager = 
		    new NewCollectionManager(
		    		taxaCollection, 
	            _nexusFileCollectionManager,
	            _nexusFileCollectionManager.outputPort()
	            );  			
    }
    

	///////////////////////////////////////////////////////////////////
	////                         protected methods                 ////

    	protected void _handleBlock() throws ParseException {
	
    	    NexusCommand dimensionsCommand = _block.getCommand("DIMENSIONS");
    	    
    	    if (dimensionsCommand != null) {
    	        
            	String taxonCountString = 
            	    dimensionsCommand.getProperty("NTAX");
            	
            	if (taxonCountString != null) {
				_taxaCollectionManager.addMetadata("taxonCount", 
				        new IntToken((Integer.parseInt(taxonCountString))));
            	}
		}
    	    
    	    _dispatchCommandHandlers();    
	}

	///////////////////////////////////////////////////////////////////
	////                         private variables                 ////
	
    private final CollectionManager _taxaCollectionManager;
    
    	///////////////////////////////////////////////////////////////////
	////                   static initializers                     ////

    	{
        _nexusCommandHandlerMap.put("TAXLABELS", new NexusCommandHandler() {
            public void handle(NexusCommand command) {
                
	            	for (Iterator i = command.argumentIterator(); 
	            			i.hasNext(); ) {
	            	    
	            	    _taxaCollectionManager.addDataToken(
	            	            new StringToken((String)i.next()));
	            	}
			}
        });
        
//        _nexusCommandHandlerMap.put("DIMENSIONS", 
//        		new NexusCommandHandler() {
        
//            public void handle(
//        				NexusCommand command) 
//    				throws IllegalActionException {
//                
//	            	String taxonCountString = command.getProperty("NTAX");
//	            	if (taxonCountString != null) {
//					_taxaCollectionManager.addMetadata(
//        				"taxonCount", Integer.valueOf(taxonCountString));
//	            	}
//			}
//        });
    }}