/* A Web Service Actor to invoke web services
 * To invoke a web service following parameters
 * are needed.
 * -URL
 * -NameSpace
 * -Method
 * -User
 * -Password
 *
 * We are adding a new Trigger to the Web Service Adapter to
 * notify its completion.
 *
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.sdm.spa;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.BooleanToken;
import ptolemy.data.StringToken;
import ptolemy.data.type.BaseType;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;
public class DummyActor extends TypedAtomicActor {

	public DummyActor(CompositeEntity aContainer, String aName)
		throws IllegalActionException, NameDuplicationException {

		super(aContainer, aName);
		result = new TypedIOPort(this, "result", false, true);
		result.setTypeEquals(BaseType.STRING);
		input = new TypedIOPort(this, "input", true, false);
		input.setTypeEquals(BaseType.STRING);

		trigger = new TypedIOPort(this, "trigger", false, true);
		trigger.setTypeEquals(BaseType.BOOLEAN);



	}

	///////////////////////////////////////////////////////////////////
	////                     ports and parameters                  ////

	// -- Part of the Actor

	public TypedIOPort result;
	public TypedIOPort input;
	public TypedIOPort trigger;

	///////////////////////////////////////////////////////////////////
	////                         public methods                    ////

	/** Initialize the PN actor.
	 *  @exception IllegalActionException If the parent class throws it.
	 */
	public void initialize() throws IllegalActionException {
		super.initialize();
		_returnValue = true;


	}


	/** Fire the actor.
	 *  @exception IllegalActionException If there is no director.
	 */
	public void fire() throws IllegalActionException {
		p_input = ((StringToken)(input.get(0))).stringValue();
		_debug(" Broadcast:Input:" + p_input);
		//Wait for 1 second;
		/*try {
			Thread.currentThread().wait(1000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}*/

		_debug(" Broadcast:Branching finished:" + p_input);
		result.broadcast(new StringToken(p_input));


		_reachedEND = true;
	}

	/** Post fire the actor. Return false to indicated that the
	 *  process has finished. If it returns true, the process will
	 *  continue indefinitely.
	 */
	public boolean postfire()  throws IllegalActionException {
		if (_reachedEND) {
			trigger.broadcast(BooleanToken.TRUE);
			return true;
		}
		//endOfLoop.broadcast(BooleanToken.FALSE);
		return true;
	}

	///////////////////////////////////////////////////////////////////
	////                         private variables                 ////

	// -- Get private copies of the parameters
	private String p_input;
	private boolean _returnValue = true;
	private boolean _reachedEND = false;

}
