/**
 *  '$RCSfile: SearchQueryGeneratorTest.java,v $'
 *  Copyright: 2000 Regents of the University of California and the
 *              National Center for Ecological Analysis and Synthesis
 *    Authors: @authors@
 *    Release: @release@
 *
 *   '$Author: ruland $'
 *     '$Date: 2005/12/15 19:25:28 $'
 * '$Revision: 1.3 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package test.org.ecoinformatics.seek.ecogrid;

import edu.ucsb.nceas.configxml.ConfigXML;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

import java.util.Hashtable;
import java.util.Vector;

import org.ecoinformatics.seek.ecogrid.SearchQueryGenerator;
import org.ecoinformatics.seek.ecogrid.exception.
                                          NoMetadataSpecificationClassException;
import org.ecoinformatics.seek.ecogrid.exception.NoSearchEndPointException;
import org.ecoinformatics.seek.ecogrid.exception.
                                              NULLEcogridConfigurationException;
import org.ecoinformatics.seek.ecogrid.exception.NULLSearchNamespaceException;
import org.ecoinformatics.util.DebugPrinter;
import org.ecoinformatics.util.Util;
import org.ecoinformatics.ecogrid.queryservice.query.ANDType;
import org.ecoinformatics.ecogrid.queryservice.query.ConditionType;
import org.ecoinformatics.ecogrid.queryservice.query.OperatorType;
import org.ecoinformatics.ecogrid.queryservice.query.ORType;
import org.ecoinformatics.ecogrid.queryservice.query.QueryType;
import org.ecoinformatics.ecogrid.queryservice.query.QueryTypeNamespace;
import org.apache.axis.types.URI;


/**
 * <p>Title: SearchScopeTest</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */

public class SearchQueryGeneratorTest  extends TestCase
{
  private static final String EML200QUERYID = "eml200-quick-search-query";
  
  public SearchQueryGeneratorTest(String name)
  {
    super(name);
  }


  /**
   * Establish a testing framework by initializing appropriate objects
   */
  public void setUp()
  {
  }

  /**
   * Release any objects after tests are complete
   */
  public void tearDown()
  {
  }

  /**
   * Create a suite of tests to be run together
   */
  public static Test suite()
  {
    TestSuite suite = new TestSuite();
    suite.addTest(new SearchQueryGeneratorTest("initialize"));
    suite.addTest(new SearchQueryGeneratorTest("getEML200Query"));
    return suite;
  }

  /**
   * Run an initial test that always passes to check that the test
   * harness is working.
   */
  public void initialize()
  {
    assertTrue(1 == 1);
  }

  public void getEML200Query()
  {
    String eml200QueryId = EML200QUERYID;
    String original      = "#value#";
    String replacement   = "soil";
    Hashtable map  = new Hashtable();
    map.put(original, replacement);
    try
    {
      SearchQueryGenerator eml200Query = new SearchQueryGenerator(eml200QueryId, map);
      QueryType query  = eml200Query.getQuery();


      String str = org.ecoinformatics.ecogrid.queryservice.util.EcogridQueryTransformer.toXMLString( query );

      System.err.println( "Query xml is: " );
      System.err.println( str );
      

      String id = query.getQueryId();
      System.err.println("The query id (after gnerate query) is "+ id);
      assertTrue(id.equals(EML200QUERYID));
      URI system = query.getSystem();
      System.err.println("The system (after gnerate query) is "+ system.toString());
      assertTrue(system.toString().equals("http://knb.ecoinformatics.org"));
      QueryTypeNamespace namespace = query.getNamespace();
      System.err.println("The namespace (after gnerate query) is "+ namespace.toString());
      assertTrue(namespace.toString().equals("eml://ecoinformatics.org/eml-2.0.0"));
      String[] titles = query.getTitle();
      String title  = titles[0];
      System.err.println("The title (after gnerate query) is "+ title);
      assertTrue(title.equals(EML200QUERYID));
      String[] returnFields = query.getReturnField();
      int length = returnFields.length;
      for (int i=0; i<length; i++)
      {
        System.err.println("The return fieled is " + returnFields[i]);
      }
      ANDType topAnd = query.getAND();
      
      ORType[] ANDConditionList = topAnd.getOR();

      ORType firstOR = ANDConditionList[0];
      
      ConditionType[] titleConditionList = firstOR.getCondition();

      ConditionType titleCondition       = titleConditionList[0];
      String titlePathStr = titleCondition.getConcept();
      assertTrue(titlePathStr.equals("dataset/title"));
      OperatorType opertor = titleCondition.getOperator();
      assertTrue(opertor.getValue().equals("LIKE"));
      assertTrue(titleCondition.get_value().equals(replacement));
      
      ORType[] orTypeList = topAnd.getOR();
      ORType or  = orTypeList[0];
      ConditionType[] conditionList = or.getCondition();
      int size = conditionList.length;
      for (int i=0; i<size; i++)
      {
        ConditionType cond = conditionList[0];
        String path = cond.getConcept();
        System.err.println(""+ path);
        OperatorType oper = cond.getOperator();
        System.err.println(""+oper);
        System.err.println(""+cond.get_value());
      }
      
      
    }
    catch(Exception e)
    {
      System.err.println("Error in getEML200Query " + e.getMessage());
      e.printStackTrace();
      assertTrue(1==2);
    }
    
    
  }


}//SearchQueryGeneratorTest

