/**

 *    '$RCSfile: GrassWebServiceInterface.java,v $'

 *    Copyright: 2004 Regents of the University of California and the

 *              National Center for Ecological Analysis and Synthesis

 *    Authors: @authors@

 *    Release: @release@

 *

 *   '$Author: berkley $'

 *   '$Date: 2004/05/03 22:14:12 $'

 *   '$Revision: 1.13 $'

 *

 * This program is free software; you can redistribute it and/or modify

 * it under the terms of the GNU General Public License as published by

 * the Free Software Foundation; either version 2 of the License, or

 * (at your option) any later version.

 *

 * This program is distributed in the hope that it will be useful,

 * but WITHOUT ANY WARRANTY; without even the implied warranty of

 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

 * GNU General Public License for more details.

 *

 * You should have received a copy of the GNU General Public License

 * along with this program; if not, write to the Free Software

 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

 *

 */

package org.ecoinformatics.seek.webservice.gis.grass;



import org.ecoinformatics.seek.webservice.gis.grass.GrassWebServiceException;



/**

 * Interface which defines all of the GRASS methods which are exposed via the

 * web services application server (GLUE -- http://themindelectric.com/).

 */

public interface GrassWebServiceInterface

{

  /**

   * imports the given arc file into the grass database and returns true

   * if the operation completed successfully.

   * @param arcFileContents the String representation of the contents of the

   * arc file

   * @throws GrassWebServiceException if the service does not complete properly

   * @return boolean true if the service completed successfully

   */

  public boolean importArcToGrass(String arcFileContents, String objectName)

    throws GrassWebServiceException;



  /**

   * exports and object from the Grass database into an arc file.  This returns

   * the contents of the arc file as a String

   * @param grassObjectName the name of the object in the grass DB to export

   * @return the arc file contents as a String, null if there was an error

   * @throws GrassWebServiceException if the service does not complete properly

   */

  public String exportGrassToArcFile(String grassObjectName)

    throws GrassWebServiceException;



  /**

   * Convert an ASCII listing of site locations into a GRASS site list file.

   * @param sites the name of the site object that the inputFile should be

   * imported into

   * @param inputFile the contents of the ascii file that should be imported

   * @param dimensions the number of dimensions represented in the file.

   * @param delimeter either 'character', 'space' or 'tab'

   * @throws GrassWebServiceException if the service does not complete properly

   * @return boolean true if the service completed successfully

   */

  public boolean importAsciiData(String sites, String inputFile,

    int dimensions, String delimiter) throws GrassWebServiceException;



  /**

   * reports attribute, label, and other information about a sites file

   * @param sites the name of the site object to get info for

   * @return String the info returned on the ascii data

   * @throws GrassWebServiceException if the service does not complete properly

   */

  public String asciiDataInfo(String sites) throws GrassWebServiceException;



  /**

   * Outputs basic information about a user-specified raster map layer.

   * @param map the name of the map object to get info for

   * @return String the info on the raster data

   * @throws GrassWebServiceException if the service does not complete properly

   */

  public String rasterDataInfo(String map) throws GrassWebServiceException;



  /**

   * Outputs basic information about a user-specified vector map layer

   * @param vector the name of the vector object to get info for

   * @return String the info on the vector data

   * @throws GrassWebServiceException if the service does not complete properly

   */

  public String vectorDataInfo(String vector) throws GrassWebServiceException;



  /**

   * Uses a GRASS sites list to produce a convex hull vector map

   * @param sites content of sites file to be input

   * @param vect name of vector object to be created

   * @throws GrassWebServiceException if the service does not complete properly

   * @return boolean true if the service completed successfully

   */

  public boolean createHull(String sites, String vect)

    throws GrassWebServiceException;



  /**

   * Creates GRASS support files for (binary) GRASS vector data.

   * @param vect name of vector object for which to create support objects

   * @throws GrassWebServiceException if the service does not complete properly

   * @return boolean true if the service completed successfully

   */

  public boolean createVectorSupportObjects(String vect)

    throws GrassWebServiceException;



  /**

   * Converts a binary GRASS vector map layer into a GRASS raster map layer.

   * @param vect name of vector object to create a raster layer of

   * @param rast name of raster object to create

   * @throws GrassWebServiceException if the service does not complete properly

   * @return boolean true if the service completed successfully

   */

  public boolean convertVectorToRaster(String vect, String rast)

    throws GrassWebServiceException;



  /**

   * Calculates a map based on the product of two rasters.  This method makes

   * the grass call 'r.mapcalc result=rast1*rast2'

   * @param rast1 the first raster input

   * @param rast2 the second raster input

   * @throws GrassWebServiceException if the service does not complete properly

   * @return boolean true if the service completed successfully

   */

  public boolean calculateMap(String rast1, String rast2)

    throws GrassWebServiceException;

}

