/**
 *    '$RCSfile: EcogridDataCacheItem.java,v $'
 *
 *     '$Author: ruland $'
 *       '$Date: 2006/01/05 15:38:06 $'
 *   '$Revision: 1.22 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 * 
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.ecoinformatics.seek.datasource;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.net.URLConnection;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ecoinformatics.seek.querybuilder.DBUIUtils;
import org.ecoinformatics.util.Config;
import org.kepler.objectmanager.cache.DataCacheManager;
import org.kepler.objectmanager.cache.DataCacheObject;
import org.w3c.dom.Node;

/**
 * This class is not as generic as the name may indicate. It is designed to get the metadata to determine 
 * where the data is stored and then does a Ecogrid "get" to the data. 
 */
public class EcogridDataCacheItem extends DataCacheObject
{
    protected String mEndPoint = null;
    private boolean mIsTarFile = false;
    private String mEntityIdentifier     = null;
    
    private static boolean lock = false;
    private boolean needSynchronize = true;
   
    private static Log log;
    static {
  	  log = LogFactory.getLog("org.ecoinformatics.seek.datasource.EcogridDataCacheItem");
    }

    /**
     * Default constructor
     *
     */
    public EcogridDataCacheItem()
    {
        super();
    }
    
     
    /**
     * 
     */
    protected String getSerialClassName()
    {
      return EcogridDataCacheItem.class.getName();
    }
    
    /* (non-Javadoc)
     * @see org.ecoinformatics.seek.datasource.DataCacheItem#doWork()
     */
    public void doWork()
    {
        log.debug("EcogridDataCacheItem - doing Work mStatus "+mStatus);
        if (mLocalFileName != null && mLocalFileName.length() > 0)
        {
            File cacheFile = new File(mLocalFileName);
            if (cacheFile.exists() && cacheFile.length() > 0)
            {
                mStatus = CACHE_OK;
            }
        }
        
        if (!isReady() && mData == null)
        {
           downloadDataFromSource();
        }        
    }
    
    /**
     * This method will download data base on distribution url in entity object
     */
    protected void downloadDataFromSource()
    {
      log.debug("download data from EcogridDataCacheItem URL : " + mResourceName);
      if (needSynchronize)
      {
          while(lock)
          {
            log.debug("sleep "+mResourceName);
            try
            {
              Thread.sleep(500);
            }
            catch(Exception e)
            {
                
            }
          }
          synchronized(this)
          {
            lock = true;
          }
         
      }
      log.debug("!!!!!!!!!!!!!Get the lock "+mResourceName);
      if (mResourceName.startsWith("http://") ||
          mResourceName.startsWith("file://")) {
        // get the data from a URL
        try {
          URL url = new URL(mResourceName);
          if (url != null) {
            URLConnection conn = url.openConnection();
            if (conn != null) {
              InputStream filestream = url.openStream();
              if (filestream != null) {
                InputStreamReader isr = new InputStreamReader(filestream);
                if (isr != null) {
                  String type = conn.getContentType();
                  
                  // Crate a new Cache Filename and write the resultsets directly to the cached file
                  mLocalFileName = createFileName();
                  File localFile = new File(mLocalFileName);
                  OutputStreamWriter osw = new OutputStreamWriter(new FileOutputStream(localFile));
                  if (osw != null)
                  {
                      char[] c = new char[1024];
                      int bread = isr.read(c, 0, 1024);
                      while (bread != -1) {
                        osw.write(c, 0, bread);
                        bread = isr.read(c, 0, 1024);
                      }
                      osw.close();
                      mStatus = CACHE_COMPLETE;
                      mHasChanged = false; // there is no need for it to be saved
                      mSize = (int)localFile.length(); // XXX this is bad thing to do
                      //System.err.println("****** Wrote "+mLocalFileName+" Size: "+mSize);
                      lock = false;
                      log.debug("release the lock!!!!!!!!!!!!!!!!!!!! for "+this.toString());
                      return;
                  }
                }
              }
            }
          }
          mStatus = CACHE_ERROR;
          log.debug("EcogridDataCacheItem - error connecting to http/file ");
        }
        catch (IOException ioe) {
          mStatus = CACHE_ERROR;
        }
        // We will use ecogrid client to handle both ecogrid and srb protocol
      }
      else if (mResourceName.startsWith("ecogrid://")) {
        // get the docid from url
        int start = mResourceName.indexOf("/", 11) + 1;
        log.debug("start: " + start);
        int end = mResourceName.indexOf("/", start);
        if (end == -1) {
          end = mResourceName.length();
        }
        log.debug("end: " + end);
        String identifier = mResourceName.substring(start, end);
        // pass this docid and get data item
        getDataItemFromEcoGrid(mEndPoint, identifier);
        saveData();
        DataCacheManager.getInstance().saveCache();
      }
      else if (mResourceName.startsWith("srb://")) {
        // get srb docid from the url
        String identifier = transformSRBurlToDocid(mResourceName);
        // reset endpoint for srb (This is hack we need to figure ou
        // elegent way to do this
        mEndPoint = Config.getValue("//ecogridService/srb/endPoint");
        // pass this docid and get data item
        log.debug("before get srb data@@@@@@@@@@@@@@@@@@@@@@@@@@");
        getDataItemFromEcoGrid(mEndPoint, identifier);
        log.debug("after get srb data@@@@@@@@@@@@@@@@@@@@@@@@@@");
        saveData();
        log.debug("after save data@@@@@@@@@@@@@@@@@@@@@@@@@@");
      }
      else {
        mStatus = CACHE_ERROR;
      }
      lock = false;
      log.debug("release the lock!!!!!!!!!!!!!!!!!!!! for "+mResourceName);
    }
    
    /**
     * 
     * @return
     */
    protected String getAttrsForXML()
    {
        return "   endpoint=\"" + mEndPoint +"\"\n";

    }
    
    /**
     * 
     * @param aNode
     */
    public boolean buildFromDOM(Node aNode)
    {
        if (aNode != null) 
        {
            mEndPoint = DBUIUtils.findAttrValue(aNode, "endpoint");
        }
        return super.buildFromDOM(aNode);
    }
    /**
     * @return Returns the mEndPoint.
     */
    public String getEndPoint()
    {
        return mEndPoint;
    }
    /**
     * @param endPoint The mEndPoint to set.
     */
    public void setEndPoint(String endPoint)
    {
        mEndPoint = endPoint;
    }
    
    /**
     * If this item is a tar file
     * @return boolean
     */
    public boolean getIsTarFile()
    {
        return mIsTarFile;
    }
    
    /**
     * Set this item is tar file or not
     * @param isTarFile boolean
     */
    public void setIsTarFile(boolean isTarFile)
    {
       mIsTarFile = isTarFile;
    }
    
   
    
    /*
     * This method will transfer a srb url to srb docid in ecogrid ecogrid
     * srb id should look like: srb://seek:/home/beam.seek/IPCC_climate/Present/ccld6190.dat
     * and correspond docid looks like:
     * srb://testuser:TESTUSER@orion.sdsc.edu/home/beam.seek/IPCC_climate/Present/ccld6190.dat
     */
    private String transformSRBurlToDocid(String srbURL)
    {
      String docid = null;
      if (srbURL == null)
      {
        return docid;
      }
      String regex = "seek:";
      srbURL = srbURL.trim();
      log.debug("The srb url is "+srbURL);
      // get user name , passwd and machine namefrom configure file
      String user = Config.getValue("//ecogridService/srb/user");
      String passwd = Config.getValue("//ecogridService/srb/passwd");
      String machineName = Config.getValue("//ecogridService/srb/machineName");
      String replacement = user+":"+passwd+"@"+machineName;
      docid = srbURL.replaceFirst(regex, replacement);
      log.debug("The srb id is " + docid);
      return docid;
    }
    
    /**
     * Set an entity to describe the data cache
     * @param mEntity Entity
     */
    public void setEntityIdentifier(String identifier ) {
        this.mEntityIdentifier = identifier;
    }
    
    /**
     * Get an entity to describe the data cache
     * @return Entity
     */
    public String getEntityIdentifier() {
        return mEntityIdentifier;
    }
    
    
   
}
