/**
 *    '$RCSfile: CreateKSWDialog.java,v $'
 *
 *     '$Author: berkley $'
 *       '$Date: 2005/12/13 23:15:16 $'
 *   '$Revision: 1.20 $'
 *
 *  For Details: http://kepler-project.org
 *
 * Copyright (c) 2005 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.kepler.ksw.gui;

import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.UIManager;

import org.ecoinformatics.seek.sms.AnnotationEngine;
import org.kepler.ksw.KSWAlreadyCachedException;
import org.kepler.ksw.KSWBuilder;
import org.kepler.moml.KSWLibraryBuilder;
import org.kepler.objectmanager.ActorMetadata;
import org.kepler.objectmanager.lsid.DuplicateLSIDException;
import org.kepler.objectmanager.lsid.KeplerLSID;
import org.kepler.objectmanager.lsid.LSIDTree;
import org.kepler.objectmanager.cache.*;

import ptolemy.actor.TypedCompositeActor;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.Entity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NamedObj;
import ptolemy.kernel.util.StringAttribute;

import com.ibm.lsid.LSIDException;
;

/**
 *@author    Chad Berkley
 *
 * This dialog lets a user create a KSW file from a model on the canvas.  It
 * lets the user select whether to create an archive ksw with all components
 * or a transport ksw with component references only.
 */
public class CreateKSWDialog extends JDialog
{
  private Frame owner;
  private Entity entity;
  private CompositeEntity actorLibrary;

  private JTextField saveToTextBox = new JTextField();
  private JTextField nameTextBox = new JTextField();
  private JRadioButton archiveRadioButton = new JRadioButton("Archive", false);
  private JRadioButton transportRadioButton = new JRadioButton("Transport", true);
  private ButtonGroup radioGroup = new ButtonGroup();
  private JButton okButton = new JButton("OK");
  private JButton cancelButton = new JButton("Cancel");
  private JButton browseButton = new JButton("Browse");
  private JComboBox categoryCombo;

  /**
   * anonymous class to handle button events
   */
  private ActionListener buttonListener =
    new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        if(e.getActionCommand().equals("OK"))
        {
          /*JOptionPane.showMessageDialog(getThis(),
             "Sorry, this doesn't currently work.",
             "alert", JOptionPane.ERROR_MESSAGE);*/
          try
          {
            File kswFile = new File(saveToTextBox.getText());
            String nameStr = nameTextBox.getText();
            if(nameStr != null)
            { //set the user's name
              entity.setName(nameStr);
            }
            else
            {
              try
              { //if the user didn't set the name, just use the old name
                //plus the revision number
                nameTextBox.setText(entity.getName() + "-" + 
                  LSIDTree.getInstance().findNextRevision(
                  new KeplerLSID(
                    ((StringAttribute)entity.getAttribute("entityId")
                    ).getExpression())).getRevision());
              }
              catch(LSIDException lside)
              {
                //do nothing...just don't set the text box default
              }
            }
            KSWBuilder kswBuilder = new KSWBuilder(entity,
              archiveRadioButton.isSelected(), kswFile);
            processKSW(kswFile);
          }
          catch(Exception ex)
          {
            JOptionPane.showMessageDialog(getThis(),
             ex.getMessage(),
             "alert", JOptionPane.ERROR_MESSAGE);
            System.out.println(ex.getMessage());
            ex.printStackTrace();
          }
          getThis().setVisible(false);
        }
        else if(e.getActionCommand().equals("Cancel"))
        {
          getThis().setVisible(false);
        }
        else if(e.getActionCommand().equals("Browse"))
        {
          JFileChooser chooser = new JFileChooser();
          int returnVal = chooser.showOpenDialog(getThis());
          if(returnVal == JFileChooser.APPROVE_OPTION)
          {
            String savePath = chooser.getSelectedFile().getAbsolutePath();
            if(!savePath.endsWith(".kar"))
            {
              savePath += ".kar";
            }
            saveToTextBox.setText(savePath);
          }
        }
      }
    };
  
  /**
   *
   */
  public CreateKSWDialog(Frame owner, Entity entity, 
    CompositeEntity actorLibrary)
  {
    this(owner, entity, actorLibrary, null);
  }
  
  /**
   * This is the default constructor
   *
   *@param  owner
   *@param  namedObj
   */
  public CreateKSWDialog(Frame owner, Entity entity, 
    CompositeEntity actorLibrary, String conceptName)
  {
    super(owner);

    this.actorLibrary = actorLibrary;
    this.entity = entity;
    this.owner = owner;
    
    //categoryCombo = new JComboBox({"a", "b", "c"});
    categoryCombo = new JComboBox(AnnotationEngine.instance().getDefaultConceptNames());
    if(conceptName != null)
    {
      for(int i=0; i<categoryCombo.getItemCount(); i++)
      {
        if(categoryCombo.getItemAt(i).toString().equals(conceptName))
        {
          categoryCombo.setSelectedIndex(i);
          break;
        }
      }
      
      //categoryCombo.setText(conceptName);
    }
    
    try
    {
      
      nameTextBox.setText(entity.getName() + "-" + 
        LSIDTree.getInstance().findNextRevision(
        new KeplerLSID(
          ((StringAttribute)entity.getAttribute("entityId")
          ).getExpression())).getRevision());
    }
    catch(LSIDException lside)
    {
      //do nothing...just don't set the text box default
    }
    catch(Exception e)
    {
    }

    archiveRadioButton.setToolTipText("Archives all components into the KAR " +
        "file which may create a large KAR file size.");
    transportRadioButton.setToolTipText("Components are referenced in the KAR " +
        "file instead of copied, which creates a smaller, but less " +
        "complete KAR file.");
    //add the radiobuttons to a group so that they are not mutually selectable
    radioGroup.add(archiveRadioButton);
    radioGroup.add(transportRadioButton);

    okButton.setActionCommand("OK");
    okButton.addActionListener(buttonListener);
    cancelButton.setActionCommand("Cancel");
    cancelButton.addActionListener(buttonListener);
    browseButton.setActionCommand("Browse");
    browseButton.addActionListener(buttonListener);

    // set title and close behavior
    this.setTitle("Save Model as KAR File");
    this.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

    JPanel pane = new JPanel();

    pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));
    pane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

    JPanel bodyPane = createBodyPane();

    // add components to pane
    pane.add(bodyPane);
    pane.add(Box.createRigidArea(new Dimension(0, 5)));

    // add the pane
    setContentPane(pane);
    // set size
    this.setSize(385, 250);
    //setResizable(false);
  }

  /**
   * Main method for testing the dialog.
   *
   *@param  args  the arguments to the program
   */
  public static void main(String[] args)
  {
    try
    {
      // a composite "wrapper"
      TypedCompositeActor swf = new TypedCompositeActor();

      // windows look and feel
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());

      CreateKSWDialog dialog = new CreateKSWDialog(null, swf, null);

      dialog.setVisible(true);
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /**
   * process the new ksw file into the actor library
   * @param kswFile the file to process
   */
  private void processKSW(File kswFile)
  {
    try
    {
      KeplerLSID[] actorLSIDs;
      CacheManager cache = CacheManager.getInstance();
      KARCacheObject kco = new KARCacheObject(kswFile);
      cache.insertObject(kco);
      try
      {
        actorLSIDs = kco.getActors();
      }
      catch(LSIDException lside)
      {
        JOptionPane.showMessageDialog(getThis(),
               "Error getting actor lsid: " + lside.getMessage(), "alert", 
               JOptionPane.ERROR_MESSAGE);
        return;
      }
      
      if(actorLSIDs != null)
      {
        System.out.println("processing kar file: " + kswFile.getAbsolutePath());
        AnnotationEngine AEngine = AnnotationEngine.instance();
        for(int j=0; j<actorLSIDs.length; j++)
        {
          ActorCacheObject aco = (ActorCacheObject)cache.getObject(actorLSIDs[j]);
          ActorMetadata am = aco.getMetadata();
          try
          { //build an entity for each actor in the ksw and add it to the actorLibrary
            NamedObj actorEntity = am.getActorAsNamedObj(actorLibrary);
            //TODO: get rid of this old lsid service and replace it with the
            //LSIDTree
            org.ecoinformatics.seek.sms.KeplerLocalLSIDService oldLSIDService = 
              org.ecoinformatics.seek.sms.KeplerLocalLSIDService.instance();
            oldLSIDService.assignLSID(am.getId(), actorEntity);
            oldLSIDService.commitChanges();
            AEngine.addActorAnnotation(am.getId(), 
              categoryCombo.getSelectedItem().toString());
            System.out.println("actor " + actorEntity.getName() +
              " loaded into the library in category " +
              categoryCombo.getSelectedItem().toString() + " with id " + 
              am.getId());
          }
          catch(Exception e)
          {
            JOptionPane.showMessageDialog(getThis(),
               "Error creating ComponentEntity from actor " +
              "metadata: " + e.getMessage(), "alert", 
               JOptionPane.ERROR_MESSAGE);
            return;
          }
        }
      }
    }
    catch(CacheException ce)
    {
      JOptionPane.showMessageDialog(getThis(),
             "Error with the cache: " + ce.getMessage(), "alert", 
             JOptionPane.ERROR_MESSAGE);
      return;
    }
    
    try
    {
      //save the actor library
      KSWLibraryBuilder.writeActorLibrary(actorLibrary);
    }
    catch(IOException ioe)
    {
      JOptionPane.showMessageDialog(getThis(),
             "Could not save the actor library: " + ioe.getMessage(), "alert", 
             JOptionPane.ERROR_MESSAGE);
      return;
    }
  }

  /**
   * create the main body pane for the dialog
   *
   */
  private JPanel createBodyPane()
  {
    JPanel bodyPanel = new JPanel();
    JPanel mainPanel = createMainPanel();
    JPanel buttonPanel = createButtonPanel();

    bodyPanel.setLayout(new BoxLayout(bodyPanel, BoxLayout.Y_AXIS));
    mainPanel.setBorder(BorderFactory.createEtchedBorder());
    //mainPanel.setPreferredSize(new Dimension(348, 50));
    bodyPanel.add(mainPanel);
    bodyPanel.add(Box.createRigidArea(new Dimension(10, 10)));
    bodyPanel.add(buttonPanel);

    return bodyPanel;
  }

  /**
   * create the panel with the main content in it
   *
   */
  private JPanel createMainPanel()
  {
    JPanel mainPanel = new JPanel();
    JPanel captionPanel = createCaptionPanel();
    JPanel inputPanel = createInputPanel();

    mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.X_AXIS));
    mainPanel.add(Box.createRigidArea(new Dimension(5, 5)));
    mainPanel.add(captionPanel);
    mainPanel.add(Box.createRigidArea(new Dimension(5, 5)));
    mainPanel.add(inputPanel);

    return mainPanel;
  }

  /**
   * create the panel with the captions for the inputs
   *
   */
  private JPanel createCaptionPanel()
  {
    JPanel captionPanel = new JPanel();
    JLabel saveToLabel = new JLabel("Save to:");
    JLabel actorNameLabel = new JLabel("Actor Name:");
    JLabel categoryLabel = new JLabel("Category:");
    JLabel kswTypeLabel = new JLabel("KAR Type:");

    captionPanel.setLayout(new BoxLayout(captionPanel, BoxLayout.Y_AXIS));
    //captionPanel.add(Box.createRigidArea(new Dimension(0, 1)));
    captionPanel.add(saveToLabel);
    captionPanel.add(Box.createRigidArea(new Dimension(0, 13)));
    captionPanel.add(actorNameLabel);
    captionPanel.add(Box.createRigidArea(new Dimension(0, 13)));
    captionPanel.add(categoryLabel);
    captionPanel.add(Box.createRigidArea(new Dimension(0, 45)));
    captionPanel.add(kswTypeLabel);
    captionPanel.add(Box.createRigidArea(new Dimension(0, 5)));

    return captionPanel;
  }

  /**
   * create the panel with the user inputs
   */
  private JPanel createInputPanel()
  {
    JPanel inputPanel = new JPanel();
    inputPanel.setLayout(new BoxLayout(inputPanel, BoxLayout.Y_AXIS));

    JPanel textBoxPanel = new JPanel();
    textBoxPanel.setLayout(new BoxLayout(textBoxPanel, BoxLayout.X_AXIS));
    textBoxPanel.add(saveToTextBox);
    textBoxPanel.add(Box.createRigidArea(new Dimension(10,0)));
    textBoxPanel.add(browseButton);
    textBoxPanel.add(Box.createRigidArea(new Dimension(10,0)));
    
    JPanel nameBoxPanel = new JPanel();
    nameBoxPanel.setLayout(new BoxLayout(nameBoxPanel, BoxLayout.X_AXIS));
    nameBoxPanel.add(nameTextBox);
    nameBoxPanel.add(Box.createRigidArea(new Dimension(10,0)));
    
    JPanel categoryPanel = new JPanel();
    categoryPanel.setLayout(new BoxLayout(categoryPanel, BoxLayout.X_AXIS));
    categoryPanel.add(categoryCombo);
    categoryPanel.add(Box.createRigidArea(new Dimension(10,0)));

    inputPanel.add(Box.createRigidArea(new Dimension(0,5)));
    inputPanel.add(textBoxPanel);
    
    inputPanel.add(Box.createRigidArea(new Dimension(0,10)));
    inputPanel.add(nameBoxPanel);
    
    inputPanel.add(Box.createRigidArea(new Dimension(0,10)));
    inputPanel.add(categoryPanel);

    inputPanel.add(Box.createRigidArea(new Dimension(0,20)));
    JPanel radioButtonPanel = createRadioPanel();
    inputPanel.add(radioButtonPanel);
    inputPanel.add(Box.createRigidArea(new Dimension(0,5)));

    return inputPanel;
  }

  /**
   * creates the panel for the radio buttons
   */
  private JPanel createRadioPanel()
  {
    JPanel radioButtonLayoutPanel = new JPanel();
    JPanel radioButtonPanel = new JPanel();
    radioButtonLayoutPanel.setLayout(new BoxLayout(radioButtonLayoutPanel, BoxLayout.X_AXIS));
    radioButtonPanel.setLayout(new BoxLayout(radioButtonPanel, BoxLayout.Y_AXIS));
    radioButtonPanel.add(archiveRadioButton);
    radioButtonPanel.add(Box.createRigidArea(new Dimension(0, 2)));
    radioButtonPanel.add(transportRadioButton);
    radioButtonLayoutPanel.add(radioButtonPanel);
    radioButtonLayoutPanel.add(Box.createRigidArea(new Dimension(140, 0)));
    return radioButtonLayoutPanel;
  }

  /**
   * create the panel with the OK and Cancel button
   *
   *@return
   */
  private JPanel createButtonPanel()
  {
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
    buttonPanel.add(Box.createRigidArea(new Dimension(200, 0)));
    buttonPanel.add(okButton);
    buttonPanel.add(Box.createRigidArea(new Dimension(5, 0)));
    buttonPanel.add(cancelButton);

    return buttonPanel;
  }

  /**
   * returns the 'this' object to the anonymous class that handles button events
   */
  private CreateKSWDialog getThis()
  {
    return this;
  }
}

