/**
 *    '$RCSfile: DataCacheObject.java,v $'
 *
 *     '$Author: ruland $'
 *       '$Date: 2006/01/05 15:38:06 $'
 *   '$Revision: 1.8 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.kepler.objectmanager.cache;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.ecoinformatics.ecogrid.queryservice.EcogridQueryClient;
import org.ecoinformatics.seek.querybuilder.DBUIUtils;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import ptolemy.util.StringUtilities;

/**
 * Represents an item in the cache
 */
public abstract class DataCacheObject implements Runnable
{
  /**
   *  Description of the Field
   */
  public final static int CACHE_OK = 0;
  /**
   *  Description of the Field
   */
  public final static int CACHE_BUSY = 1;
  /**
   *  Description of the Field
   */
  public final static int CACHE_ERROR = 2;
  /**
   *  Description of the Field
   */
  public final static int CACHE_COMPLETE = 3;
  /**
   *  Description of the Field
   */
  public final static int CACHE_EMPTY = 4;
  /**
   *  Description of the Field
   */
  public final static String FACTORY = "Factory";
  private static boolean lockEcoGrid = false;

  /**
   *  Description of the Field
   */
  public Thread mThread = null;

  /**
   *  Description of the Field
   */
  private static Log log;
  static {
	  log = LogFactory.getLog("org.kepler.objectmanager.cache.DataCacheObject");
  }
  /**
   *  Description of the Field
   */
  protected byte[] mData = null;
  /**
   *  Description of the Field
   */
  protected Date mCreatedDate = new Date();
  /**
   *  Description of the Field
   */
  protected String mName = null;
  /**
   *  Description of the Field
   */
  protected String mResourceName = null;
  /**
   *  Description of the Field
   */
  protected String mLocalFileName = null;
  /**
   *  Description of the Field
   */
  protected int mSize = 0;
  /**
   *  Description of the Field
   */
  protected Hashtable mAttrs = new Hashtable();

  /**
   *  Description of the Field
   */
  protected Vector mListeners = new Vector();
  /**
   *  Description of the Field
   */
  protected Vector mNotifierList = null;

  /**
   *  Description of the Field
   */
  protected boolean mIsLoading = false;
  /**
   *  Description of the Field
   */
  protected int mCurrInx = 0;
  /**
   *  Description of the Field
   */
  protected int mStatus = CACHE_EMPTY;
  /**
   *  Description of the Field
   */
  protected boolean mHasChanged = true;
  /**
   *  Description of the Field
   */
  protected boolean mDoingSave = false;
  /**
   *  Description of the Field
   */
  protected boolean mCanceled = false;
  /**
   *  Description of the Field
   */
  protected boolean needSynchronize = true;

  /**
   *  Integer index used to ensure filenames are unique.
   */
  private static int index = 0;
  
  private boolean hasListener = false;
  
  /**
   * Constructor
   */
  public DataCacheObject()
  {
  }

  /**
   * Sets the data into the cache item
   *
   *@param  aData  the new data byte array
   */
  public void setData(byte[] aData)
  {
    mData = aData;
    if(mData != null)
    {
      mSize = mData.length;
    }
    mHasChanged = true;
  }

  /**
   *@param  createdDate  The mCreatedDate to set.
   */
  public void setCreatedDate(Date createdDate)
  {
    mCreatedDate = createdDate;
  }

  /**
   *@param  localFileName  The mLocalFileName to set.
   */
  public void setLocalFileName(String localFileName)
  {
    mLocalFileName = localFileName;
  }

  /**
   *@param  name  The mName to set.
   */
  public void setName(String name)
  {
    mName = name;
  }

  /**
   *@param  resourceName  The mResourceName to set.
   */
  public void setResourceName(String resourceName)
  {
    mResourceName = resourceName;
  }

  /**
   * Sets the new status
   *
   *@param  aStatus  The new status value
   */
  public void setStatus(int aStatus)
  {
    mStatus = aStatus;
  }

  /**
   * Sets an attribute
   *
   *@param  aName   the name of the attr
   *@param  aValue  the value
   */
  public void setAttr(String aName, String aValue)
  {
    if(aName != null && aValue != null)
    {
      mHasChanged = true;
      mAttrs.put(aName, aValue);
    }
  }

  /**
   * Returns the data as a byte array
   *
   *@return
   */
  public byte[] getData()
  {
    if(mData == null)
    {
      loadData();
    }
    return mData;
  }
  
  /**
   * Method to get if the object has listener. This value is for a workflow
   * which has two copies of same data source
   * @return
   */
  public boolean hasListener()
  {
      return hasListener;
  }
  
  /**
   * Set the property of hasListener
   * @param hasListener
   */
  public void setHasListener(boolean hasListener)
  {
      this.hasListener = hasListener;
  }

  /**
   * Return the data as an InputStream.  The data is read from the cached
   * file on disk, so this method assumes the cache item has already been
   * retrieved. If not, or if there is an error, then the return value
   * will be null.
   *
   *
   *@return    InputStream representing the data
   */
  public InputStream getDataInputStream()
  {
    BufferedInputStream bis = null;

    if(mLocalFileName != null && mLocalFileName.length() > 0)
    {
      try
      {
        File file = new File(mLocalFileName);

        if(file != null && file.exists())
        {
          FileInputStream fis = new FileInputStream(file);

          if(fis != null)
          {
            bis = new BufferedInputStream(fis);
          }
        }
      }
      catch(Exception e)
      {
        System.err.println(e);
      }
    }
    else
    {
      log.debug("loadData - mLocalFileName was null: \n  " + mName
           + "  \n" + mResourceName + "  \n" + mLocalFileName);
    }
    return bis;
  }

  /**
   * Return the status of getting the data
   *
   *
   *@return
   */
  public boolean isReady()
  {
    return mStatus == CACHE_OK || mStatus == CACHE_COMPLETE;
  }

  /**
   * Returns whether it is busy getting the data
   *
   *
   *@return
   */
  public boolean isBusy()
  {
    return mStatus == CACHE_BUSY;
  }

  /**
   *@return    Returns the mCreatedDate.
   */
  public Date getCreatedDate()
  {
    return mCreatedDate;
  }

  /**
   *@return    Returns the mLocalFileName.
   */
  public String getLocalFileName()
  {
    return mLocalFileName;
  }

  /**
   *@return    Returns the mLocalFileName.
   */
  public String getFileName()
  {
    int inx = mLocalFileName.lastIndexOf(File.separator);

    if(inx > -1)
    {
      return mLocalFileName.substring(inx + 1);
    }
    return mLocalFileName;
  }

  /**
   *@return    Returns the mName.
   */
  public String getName()
  {
    return mName;
  }

  /**
   *@return    Returns the mResourceName.
   */
  public String getResourceName()
  {
    return mResourceName;
  }

  /**
   *@return    Returns the mStatus.
   */
  public int getStatus()
  {
    return mStatus;
  }

  /**
   *@return    Returns the mSize.
   */
  public int getSize()
  {
    return mSize;
  }

  /**
   * Returns a named attribute of the item
   *
   *@param  aName  the attribute name
   *@return        the value
   */
  public String getAttr(String aName)
  {
    if(aName != null)
    {
      return (String)mAttrs.get(aName);
    }
    return null;
  }

  /**
   *@return    a vector of the obbjects that are hashed
   */
  public Hashtable getAttrs()
  {
    return mAttrs;
  }


  /**
   * Add a listenero
   *
   *@param  aListener  the listener to add
   */
  public void addListener(DataCacheListener aListener)
  {
    if(aListener != null)
    {
      mListeners.addElement(aListener);
    }
  }

  /**
   * Removes a listener
   *
   *@param  aListener  the listener to remove
   */
  public void removeListener(DataCacheListener aListener)
  {
    if(aListener != null)
    {
      mListeners.removeElement(aListener);
    }
  }

  /**
   * Removes all the listeners
   */
  public void removeAllListeners()
  {
    mListeners.clear();
  }

  /**
   * Notifies the listener that the "getting" of the data has completed
   */
  public synchronized void notifyListeners()
  {
    mNotifierList = new Vector(mListeners);

    for(Enumeration e = mNotifierList.elements(); e.hasMoreElements(); )
    {
      DataCacheListener l = (DataCacheListener)e.nextElement();

      //System.err.println("notifyListeners: "+this + "  " +l);
      l.complete(this);
    }
  }

  /**
   * Refreshes the data from the original source
   *
   *@param  aListener
   */
  public void refresh(DataCacheListener aListener)
  {
    clear();
    mLocalFileName = createFileName();
    //System.err.println("Refresh Name "+mName + "has new file name"+mLocalFileName);
    addListener(aListener);
    mCreatedDate = new Date();
    mData = null;
    mHasChanged = true;
    start();
  }

  /**
   *  Description of the Method
   */
  public void reset()
  {
    clear();
    mStatus = CACHE_EMPTY;
    mLocalFileName = null;
  }

  /**
   * Clear the dat from this cache item
   */
  public void clear()
  {
    if(mLocalFileName != null && mLocalFileName.length() > 0)
    {
      File file = new File(mLocalFileName);

      if(file != null && file.exists())
      {
        file.delete();
      }
    }
    mData = null;
    removeAllListeners();
  }

  /**
   * Abstract method for actually getting the data while in the thread
   */
  public abstract void doWork();

  /**
   * Returns the XML representing this item
   *
   *@return
   */
  public String toXMLString()
  {
    StringBuffer strBuf = new StringBuffer();

    if(isOKToPersist())
    {
      strBuf.append("  <item\n");
      strBuf.append("   classname=\"" + getSerialClassName() + "\"\n");
      strBuf.append("   name=\"" + mName + "\"\n");
      strBuf.append("   resname=\"" + (mResourceName != null ? StringUtilities.escapeForXML(mResourceName) : "") + "\"\n");
      strBuf.append("   date=\"" + (mCreatedDate != null ? mCreatedDate.getTime() : 0) + "\"\n");
      strBuf.append("   fileName=\"" + mLocalFileName + "\"\n");
      strBuf.append("   " + getAttrsForXML());
      strBuf.append("   size=\"" + mSize + "\"");

      String contents = getItemContentsForXML();

      if(contents != null)
      {
        strBuf.append(">\n");
        strBuf.append(contents);
        strBuf.append("  </item>\n");
      }
      else
      {
        strBuf.append("/>\n");
      }
    }
    return strBuf.toString();
  }
  
  /**
   * return a string representation of this datacacheobject
   */
  public String toString()
  {
    StringBuffer sb = new StringBuffer();
    sb.append("{");
    sb.append("classname = " + getSerialClassName() + "\n");
    sb.append("name = " + mName + "\n");
    //sb.append("resname = " + (mCreatedDate != null ? mCreatedDate.getTime() : 0) + "\n");
    sb.append("resname = " + mResourceName + "\n");
    sb.append("date = " + (mCreatedDate != null ? mCreatedDate.getTime() : 0) + "\n");
    sb.append("fileName = " + mLocalFileName + "\n");
    sb.append("size = " + mSize);
    sb.append("}");
    return sb.toString();
  }

  /**
   * Fills in this node from an XML Node in the DOM tree
   *
   *@param  aNode  the DOM node representing this object
   *@return
   */
  public boolean buildFromDOM(Node aNode)
  {

    if(aNode != null)
    {
      mName = DBUIUtils.findAttrValue(aNode, "name");
      mResourceName = StringUtilities.unescapeForXML(
        DBUIUtils.findAttrValue(aNode, "resname"));
      mLocalFileName = DBUIUtils.findAttrValue(aNode, "fileName");
      try
      {
        long time = Long.parseLong(DBUIUtils.findAttrValue(aNode, "date"));

        mCreatedDate = new Date(time);
      }
      catch(Exception e)
      {
        return false;
      }
      mSize = getIntFromStr(DBUIUtils.findAttrValue(aNode, "size"));
      if(mSize == Integer.MAX_VALUE)
      {
        mSize = 0;
        return false;
      }

      Node attrsNode = DBUIUtils.findNode(aNode, "attributes");

      if(attrsNode != null)
      {
        NodeList list = attrsNode.getChildNodes();

        if(list != null)
        {
          for(int i = 0; i < list.getLength(); i++)
          {
            Node child = list.item(i);

            if(child.getNodeName().equals("attr"))
            {
              String name = DBUIUtils.findAttrValue(child, "name");
              String value = StringUtilities.unescapeForXML(
                DBUIUtils.findAttrValue(child, "value"));

              mAttrs.put(name, value);
            }
          }
        }
      }
    }
    mHasChanged = false;
    return true;
  }

  /**
   * Loads the data from the cache (a file)
   *
   *@return    whether it was able to load the data
   */
  public boolean loadData()
  {
    if(mLocalFileName != null && mLocalFileName.length() > 0)
    {
      try
      {
        File file = new File(mLocalFileName);

        if(file != null && file.exists())
        {
          long size = file.length();

          if(size <= Integer.MAX_VALUE)
          {
            // XXX this could be bad for large datasets

            int iSize = (int)size;

            if(mSize > 0)
            {
              if(mData == null || iSize != mSize)
              {
                mData = new byte[mSize];
              }
            }
            else
            {
              log.debug("loadData() - mSize == 0");
              return false;
            }

            FileInputStream fis = new FileInputStream(file);

            if(fis != null)
            {
              BufferedInputStream bis = new BufferedInputStream(fis);

              if(bis != null)
              {
                int retSize = bis.read(mData, 0, iSize);

                if(retSize != iSize)
                {
                  log.debug("loadData - Wrong amount of data read: \n  " +
                    mName + "  \n" + mResourceName + "  \n" + mLocalFileName);
                }
                bis.close();
                mStatus = CACHE_OK;
                log.debug("DataCacheObject - Data was loaded: " + mName + "  \n" +
                  mResourceName + "  \n" + mLocalFileName);
              }
            }
          }
          else
          {
            log.debug("loadData - Too Much Data for byte array: \n  " + mName +
              "  \n" + mResourceName + "  \n" + mLocalFileName);
          }
        }

      }
      catch(Exception e)
      {
        System.err.println(e);
        return false;
      }
      return true;
    }
    else
    {
      log.debug("loadData - mLocalFileName was null: \n  " + mName + "  \n" +
        mResourceName + "  \n" + mLocalFileName);
      return false;
    }
  }

  /**
   * Save the data out to a file
   *
   *@return
   */
  public boolean saveData()
  {
    if(!mDoingSave && mData != null && mHasChanged)
    {
      //mDoingSave = true;
      try
      {
        if(mLocalFileName == null || mLocalFileName.length() == 0)
        {
          mLocalFileName = createFileName();
          //System.err.println("Name "+mName + " has new file name "+mLocalFileName);
        }
        //System.err.println("**** Saving "+mLocalFileName+" "+mData.length);
        if(mData != null && mData.length > 0)
        {
          File file = new File(mLocalFileName);
          FileOutputStream fos = new FileOutputStream(file);
          BufferedOutputStream bos = new BufferedOutputStream(fos);

          bos.write(mData);
          bos.close();
          mHasChanged = false;
        }

      }
      catch(Exception e)
      {
        System.err.println(e);
        return false;
      }
      mDoingSave = false;
    }
    return true;
  }

  //----------------------------------------------------------------
  //-- Runnable Interface
  //----------------------------------------------------------------
  /**
   *  Description of the Method
   */
  public void start()
  {
    if(mThread == null)
    {
      mThread = new Thread(this);
      mThread.setPriority(Thread.MIN_PRIORITY);
      mThread.setName(mName);
      mThread.start();
    }
  }

  /**
   */
  public synchronized void stop()
  {
    if(mThread != null)
    {
      mThread.interrupt();
    }
    mThread = null;
    mCanceled = true;
    notifyAll();
  }

  /**
   */
  public void run()
  {
    Thread me = Thread.currentThread();

    if(!isReady())
    {
      mStatus = CACHE_BUSY;

      while(mThread == me && mStatus == CACHE_BUSY)
      {
        // do work here
        doWork();

      }
    }

    // we already save the cache in dowork() for ecogrid query cache
    // BBBBBBBBBBBBB  will be taken care of later.
    // if(!(this instanceof EcogridQueryDataCacheItem))
    {
      //System.out.println("need to store cache in run for non query item");
      DataCacheManager.getInstance().saveCache();
    }
    log.debug("run - Done With Work.");
    {
      //if (!mCanceled)

      notifyListeners();
    }

    mThread = null;
  }

  /**
   *  Description of the Method
   *
   *@param  aName
   */
  public void removeAttr(String aName)
  {
    if(aName != null)
    {
      mAttrs.remove(mAttrs.get(aName));
    }
  }

  /**
   * Returns the class name of the object that should be created to handle this type of data
   *
   *@return
   */
  protected abstract String getSerialClassName();

  /**
   * Returns any extra attributes a derived class may have that needs to augment the XML
   *
   *@return
   */
  protected abstract String getAttrsForXML();

  /**
   * Returns any children nodes of the item
   *
   *@return    string of XML representing the children nodes
   */
  protected String getItemContentsForXML()
  {
    if(mAttrs.size() > 0)
    {
      StringBuffer strBuf = new StringBuffer();

      strBuf.append("     <attributes>\n");
      for(Enumeration e = mAttrs.keys(); e.hasMoreElements(); )
      {
        String key = (String)e.nextElement();
        String value = (String)mAttrs.get(key);

        if(value != null)
        {
          strBuf.append("       <attr name=\"" + key + "\" value=\"" + StringUtilities.escapeForXML(value) + "\"/>\n");
        }
      }
      strBuf.append("     </attributes>\n");
      return strBuf.toString();
    }
    return null;
  }

  /**
   * An overridable method for enabling the item to determine if
   * it can be persisted into the cache
   *
   *@return    true if it can be cached, false if not
   */
  protected boolean isOKToPersist()
  {
    return (mLocalFileName != null &&
        mSize > 0 &&
        mName.length() > 0 &&
        mResourceName.length() > 0);
  }

  /**
   * Returns an int from a string
   *
   *@param  aVal
   *@return
   */
  protected int getIntFromStr(String aVal)
  {
    int i = Integer.MAX_VALUE;

    if(aVal != null)
    {
      try
      {
        i = Integer.parseInt(aVal);
      }
      catch(Exception e)
      {
      }
    }
    return i;
  }

  /*
   *  This method will get data from ecogrid server base on given
   *  docid. This method will handle the distribution url is ecogrid or
   *  srb protocol
   */
  /**
   *  Gets the dataItemFromEcoGrid attribute of the DataCacheObject object
   *
   *@param  endPoint
   *@param  identifier
   */
  protected void getDataItemFromEcoGrid(String endPoint, String identifier)
  {

    // create a ecogrid client object and get the full record from the
    // client
    log.debug("Get " + identifier + " from " + endPoint);
    if(needSynchronize)
    {
      while(lockEcoGrid)
      {
        log.debug("sleep " + identifier);
        try
        {
          Thread.sleep(500);
        }
        catch(Exception e)
        {

        }
      }
      synchronized(this)
      {
        lockEcoGrid = true;
      }

    }
    log.debug("get lock for " + identifier);
    try
    {
      //fatory
      log.debug("This is instance pattern");

      EcogridQueryClient ecogridClient = new EcogridQueryClient(endPoint);

      log.debug("Get from EcoGrid: " + identifier);
      mData = ecogridClient.get(identifier);

    }
    catch(Exception ee)
    {
      mStatus = CACHE_ERROR;
      log.debug("EcogridDataCacheItem - error connecting to Ecogrid ");

    }

    if(mData != null)
    {
      mSize = mData.length;
      log.debug("Bytes returned: " + mData.length);
      mStatus = CACHE_COMPLETE;
    }
    else
    {
      mStatus = CACHE_ERROR;
      log.debug("EcogridDataCacheItem - Ecogrid returned null data");
    }
    lockEcoGrid = false;
    log.debug("realse lock from " + identifier);

  }

  /**
   * Creates a new unique data file based on the time
   *
   *@return    the name of the file
   */
  protected final String createFileName()
  {

    String fileName = DataCacheManager.getInstance().getDataPath();
    File dir = new File(fileName);

    dir.mkdirs();
    fileName += Long.toString(mCreatedDate.getTime()) + "-" + Long.toString(getIndexNumber()) + ".dat";

    File file = new File(fileName);

    //System.err.println("****** createFileName "+fileName);
    return file.getAbsolutePath();
  }
  
  /**
   * Returns a number between 0-99.
   * On faster machines, the file name generated in createFileName based only
   * on the mCreatedDate is not always unique.  This method is synchronized and
   * returns sequential integers between [0,99] (inclusive) which are appended to
   * the file creation date stamp.  This should ensure that filenames are unique.
   * Note: there is no guarantee.  If more than 100 DataCacheObjects are created in
   * the same milisecond, then there could be problems.  Since this class will be
   * removed in the near future, additional effort is not needed.
   * @return
   */
  private static synchronized int getIndexNumber() {
      if ( index > 99 ) {
          index = 0;
      }
      return index++;
  }
}

